--
-- VIEWS
--
BEGIN;
SET ROLE :db_adm;

CREATE OR REPLACE VIEW page_einverstaendnis_view AS SELECT
    master_tbl.id AS id,
    master_tbl.einverstaendniserklaerung AS einverstaendniserklaerung
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_einverstaendnis_update_rule AS ON UPDATE TO page_einverstaendnis_view
DO INSTEAD UPDATE master_tbl SET
    einverstaendniserklaerung = NEW.einverstaendniserklaerung
WHERE id = NEW.id;

GRANT SELECT ON page_einverstaendnis_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_einverstaendnis_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_einverstaendnis_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_einverstaendnis_delete_rule AS ON DELETE TO page_einverstaendnis_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_einverstaendnis_insert_rule AS ON INSERT TO page_einverstaendnis_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_0_view AS SELECT
    master_tbl.id AS id,
    master_tbl.fkz AS fkz,
    master_tbl.name_kompetenzagentur AS name_kompetenzagentur,
    master_tbl.vorname AS vorname,
    master_tbl.name AS name,
    master_tbl.erstgespraech AS erstgespraech,
    master_tbl.zugang_kompetenzagentur AS zugang_kompetenzagentur,
    master_tbl.kundennummer AS kundennummer,
    master_tbl.schulpflichtig AS schulpflichtig,
    master_tbl.noch_schueler AS noch_schueler,
    master_tbl.bei_eintritt_ist_der_jugendliche AS bei_eintritt_ist_der_jugendliche,
    master_tbl.rechtlicher_kontext AS rechtlicher_kontext,
    master_tbl.schnittstelle_zu AS schnittstelle_zu
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_0_update_rule AS ON UPDATE TO page_0_view
DO INSTEAD UPDATE master_tbl SET
    fkz = NEW.fkz,
    name_kompetenzagentur = NEW.name_kompetenzagentur,
    vorname = NEW.vorname,
    name = NEW.name,
    erstgespraech = NEW.erstgespraech,
    zugang_kompetenzagentur = NEW.zugang_kompetenzagentur,
    kundennummer = NEW.kundennummer,
    schulpflichtig = NEW.schulpflichtig,
    noch_schueler = NEW.noch_schueler,
    bei_eintritt_ist_der_jugendliche = NEW.bei_eintritt_ist_der_jugendliche,
    rechtlicher_kontext = NEW.rechtlicher_kontext,
    schnittstelle_zu = NEW.schnittstelle_zu
WHERE id = NEW.id;

GRANT SELECT ON page_0_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_0_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_0_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_0_delete_rule AS ON DELETE TO page_0_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_0_insert_rule AS ON INSERT TO page_0_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_25_view AS SELECT
    master_tbl.id AS id,
    master_tbl.kontaktdaten_arge AS kontaktdaten_arge,
    master_tbl.kundennummer_arge AS kundennummer_arge,
    master_tbl.kontaktdaten_agentur_fuer_arbeit AS kontaktdaten_agentur_fuer_arbeit,
    master_tbl.kundennummer_agentur_fuer_arbeit AS kundennummer_agentur_fuer_arbeit,
    master_tbl.kundennummer_jugendamt AS kundennummer_jugendamt,
    master_tbl.kontaktdaten_jugendamt AS kontaktdaten_jugendamt,
    master_tbl.kontaktdaten_schule AS kontaktdaten_schule,
    master_tbl.schulname AS schulname,
    master_tbl.kontaktdaten_gerichtshilfe AS kontaktdaten_gerichtshilfe,
    master_tbl.kundennummer_gerichtshilfe AS kundennummer_gerichtshilfe,
    master_tbl.kontaktdaten_weitere_institutionen AS kontaktdaten_weitere_institutionen
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_25_update_rule AS ON UPDATE TO page_25_view
DO INSTEAD UPDATE master_tbl SET
    kontaktdaten_arge = NEW.kontaktdaten_arge,
    kundennummer_arge = NEW.kundennummer_arge,
    kontaktdaten_agentur_fuer_arbeit = NEW.kontaktdaten_agentur_fuer_arbeit,
    kundennummer_agentur_fuer_arbeit = NEW.kundennummer_agentur_fuer_arbeit,
    kundennummer_jugendamt = NEW.kundennummer_jugendamt,
    kontaktdaten_jugendamt = NEW.kontaktdaten_jugendamt,
    kontaktdaten_schule = NEW.kontaktdaten_schule,
    schulname = NEW.schulname,
    kontaktdaten_gerichtshilfe = NEW.kontaktdaten_gerichtshilfe,
    kundennummer_gerichtshilfe = NEW.kundennummer_gerichtshilfe,
    kontaktdaten_weitere_institutionen = NEW.kontaktdaten_weitere_institutionen
WHERE id = NEW.id;

GRANT SELECT ON page_25_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_25_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_25_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_25_delete_rule AS ON DELETE TO page_25_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_25_insert_rule AS ON INSERT TO page_25_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_1_view AS SELECT
    master_tbl.id AS id,
    master_tbl.vorname_1 AS vorname_1,
    master_tbl.name_1 AS name_1,
    master_tbl.geschlecht AS geschlecht,
    master_tbl.geburtsdatum AS geburtsdatum,
    master_tbl.familienstand AS familienstand,
    master_tbl.strasse AS strasse,
    master_tbl.plz AS plz,
    master_tbl.wohnort AS wohnort,
    master_tbl.groesse_wohnort AS groesse_wohnort,
    master_tbl.vorwahl AS vorwahl,
    master_tbl.telefon_1 AS telefon_1,
    master_tbl.mobiltelefon AS mobiltelefon,
    master_tbl.email AS email,
    master_tbl.telefon_sonstige AS telefon_sonstige,
    master_tbl.zweiter_wohnort AS zweiter_wohnort,
    master_tbl.geburtsland AS geburtsland,
    master_tbl.geburtsland_sonstige_nennung AS geburtsland_sonstige_nennung,
    master_tbl.muttersprache AS muttersprache,
    master_tbl.muttersprache_sonstige_nennung AS muttersprache_sonstige_nennung,
    master_tbl.staatsangehoerigkeit AS staatsangehoerigkeit,
    master_tbl.staatsangehoerigkeit_sonstige_nennung AS staatsangehoerigkeit_sonstige_nennung,
    master_tbl.aufenthaltsstatus AS aufenthaltsstatus,
    master_tbl.aufenthaltsdauer AS aufenthaltsdauer,
    master_tbl.vater AS vater,
    master_tbl.mutter AS mutter
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_1_update_rule AS ON UPDATE TO page_1_view
DO INSTEAD UPDATE master_tbl SET
    vorname_1 = NEW.vorname_1,
    name_1 = NEW.name_1,
    geschlecht = NEW.geschlecht,
    geburtsdatum = NEW.geburtsdatum,
    familienstand = NEW.familienstand,
    strasse = NEW.strasse,
    plz = NEW.plz,
    wohnort = NEW.wohnort,
    groesse_wohnort = NEW.groesse_wohnort,
    vorwahl = NEW.vorwahl,
    telefon_1 = NEW.telefon_1,
    mobiltelefon = NEW.mobiltelefon,
    email = NEW.email,
    telefon_sonstige = NEW.telefon_sonstige,
    zweiter_wohnort = NEW.zweiter_wohnort,
    geburtsland = NEW.geburtsland,
    geburtsland_sonstige_nennung = NEW.geburtsland_sonstige_nennung,
    muttersprache = NEW.muttersprache,
    muttersprache_sonstige_nennung = NEW.muttersprache_sonstige_nennung,
    staatsangehoerigkeit = NEW.staatsangehoerigkeit,
    staatsangehoerigkeit_sonstige_nennung = NEW.staatsangehoerigkeit_sonstige_nennung,
    aufenthaltsstatus = NEW.aufenthaltsstatus,
    aufenthaltsdauer = NEW.aufenthaltsdauer,
    vater = NEW.vater,
    mutter = NEW.mutter
WHERE id = NEW.id;

GRANT SELECT ON page_1_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_1_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_1_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_1_delete_rule AS ON DELETE TO page_1_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_1_insert_rule AS ON INSERT TO page_1_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_2_view AS SELECT
    master_tbl.id AS id,
    master_tbl.familiaere_situation_1 AS familiaere_situation_1,
    master_tbl.vorname_2 AS vorname_2,
    master_tbl.name_2 AS name_2,
    master_tbl.vorwahl_1 AS vorwahl_1,
    master_tbl.telefon_1_1 AS telefon_1_1,
    master_tbl.mobiltelefon_1 AS mobiltelefon_1,
    master_tbl.email_1 AS email_1,
    master_tbl.geschwister AS geschwister,
    master_tbl.anzahl_kinder AS anzahl_kinder,
    master_tbl.haushalt_jugendlicher AS haushalt_jugendlicher,
    master_tbl.alleinerziehend AS alleinerziehend,
    master_tbl.familie_selbststaendig AS familie_selbststaendig,
    master_tbl.familie_staatliche_unterstuetzung AS familie_staatliche_unterstuetzung,
    master_tbl.jugendlicher_selbststaendig AS jugendlicher_selbststaendig,
    master_tbl.jugendlicher_staatliche_unterstuetzung AS jugendlicher_staatliche_unterstuetzung,
    master_tbl.jugendlicher_unterstuetzung_soziales_umfeld AS jugendlicher_unterstuetzung_soziales_umfeld,
    master_tbl.wohnaufenthalt AS wohnaufenthalt
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_2_update_rule AS ON UPDATE TO page_2_view
DO INSTEAD UPDATE master_tbl SET
    familiaere_situation_1 = NEW.familiaere_situation_1,
    vorname_2 = NEW.vorname_2,
    name_2 = NEW.name_2,
    vorwahl_1 = NEW.vorwahl_1,
    telefon_1_1 = NEW.telefon_1_1,
    mobiltelefon_1 = NEW.mobiltelefon_1,
    email_1 = NEW.email_1,
    geschwister = NEW.geschwister,
    anzahl_kinder = NEW.anzahl_kinder,
    haushalt_jugendlicher = NEW.haushalt_jugendlicher,
    alleinerziehend = NEW.alleinerziehend,
    familie_selbststaendig = NEW.familie_selbststaendig,
    familie_staatliche_unterstuetzung = NEW.familie_staatliche_unterstuetzung,
    jugendlicher_selbststaendig = NEW.jugendlicher_selbststaendig,
    jugendlicher_staatliche_unterstuetzung = NEW.jugendlicher_staatliche_unterstuetzung,
    jugendlicher_unterstuetzung_soziales_umfeld = NEW.jugendlicher_unterstuetzung_soziales_umfeld,
    wohnaufenthalt = NEW.wohnaufenthalt
WHERE id = NEW.id;

GRANT SELECT ON page_2_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_2_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_2_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_2_delete_rule AS ON DELETE TO page_2_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_2_insert_rule AS ON INSERT TO page_2_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_3_view AS SELECT
    master_tbl.id AS id,
    master_tbl.arzt_gutachten_physisch AS arzt_gutachten_physisch,
    master_tbl.physische_beeintraechtigungen AS physische_beeintraechtigungen,
    master_tbl.physische_beeintraechtigungen_text AS physische_beeintraechtigungen_text,
    master_tbl.arzt_gutachten_psychisch AS arzt_gutachten_psychisch,
    master_tbl.psychische_beeintraechtigungen AS psychische_beeintraechtigungen,
    master_tbl.psychische_beeintraechtigungen_text AS psychische_beeintraechtigungen_text
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_3_update_rule AS ON UPDATE TO page_3_view
DO INSTEAD UPDATE master_tbl SET
    arzt_gutachten_physisch = NEW.arzt_gutachten_physisch,
    physische_beeintraechtigungen = NEW.physische_beeintraechtigungen,
    physische_beeintraechtigungen_text = NEW.physische_beeintraechtigungen_text,
    arzt_gutachten_psychisch = NEW.arzt_gutachten_psychisch,
    psychische_beeintraechtigungen = NEW.psychische_beeintraechtigungen,
    psychische_beeintraechtigungen_text = NEW.psychische_beeintraechtigungen_text
WHERE id = NEW.id;

GRANT SELECT ON page_3_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_3_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_3_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_3_delete_rule AS ON DELETE TO page_3_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_3_insert_rule AS ON INSERT TO page_3_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_4_view AS SELECT
    master_tbl.id AS id,
    master_tbl.mutter_2 AS mutter_2,
    master_tbl.vater_2 AS vater_2,
    master_tbl.grosseltern AS grosseltern,
    master_tbl.geschwister_2 AS geschwister_2,
    master_tbl.pflegefamilie_1 AS pflegefamilie_1,
    master_tbl.stiefmutter AS stiefmutter,
    master_tbl.stiefvater AS stiefvater,
    master_tbl.verwandte_1 AS verwandte_1,
    master_tbl.partner AS partner,
    master_tbl.heimbetreuerin AS heimbetreuerin,
    master_tbl.betreuerin AS betreuerin,
    master_tbl.lehrerin AS lehrerin,
    master_tbl.mitbewohner AS mitbewohner,
    master_tbl.partner_mutter AS partner_mutter,
    master_tbl.partner_vater AS partner_vater,
    master_tbl.freund AS freund,
    master_tbl.keine_bezugsperson AS keine_bezugsperson,
    master_tbl.hobbies_1 AS hobbies_1,
    master_tbl.hobbies_text AS hobbies_text,
    master_tbl.mitarbeit_organisationen AS mitarbeit_organisationen,
    master_tbl.organisationen_vereine_text AS organisationen_vereine_text,
    master_tbl.sonstiges_1 AS sonstiges_1,
    master_tbl.sonstiges_text AS sonstiges_text,
    master_tbl.clique AS clique,
    master_tbl.freundin AS freundin,
    master_tbl.parterin AS parterin
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_4_update_rule AS ON UPDATE TO page_4_view
DO INSTEAD UPDATE master_tbl SET
    mutter_2 = NEW.mutter_2,
    vater_2 = NEW.vater_2,
    grosseltern = NEW.grosseltern,
    geschwister_2 = NEW.geschwister_2,
    pflegefamilie_1 = NEW.pflegefamilie_1,
    stiefmutter = NEW.stiefmutter,
    stiefvater = NEW.stiefvater,
    verwandte_1 = NEW.verwandte_1,
    partner = NEW.partner,
    heimbetreuerin = NEW.heimbetreuerin,
    betreuerin = NEW.betreuerin,
    lehrerin = NEW.lehrerin,
    mitbewohner = NEW.mitbewohner,
    partner_mutter = NEW.partner_mutter,
    partner_vater = NEW.partner_vater,
    freund = NEW.freund,
    keine_bezugsperson = NEW.keine_bezugsperson,
    hobbies_1 = NEW.hobbies_1,
    hobbies_text = NEW.hobbies_text,
    mitarbeit_organisationen = NEW.mitarbeit_organisationen,
    organisationen_vereine_text = NEW.organisationen_vereine_text,
    sonstiges_1 = NEW.sonstiges_1,
    sonstiges_text = NEW.sonstiges_text,
    clique = NEW.clique,
    freundin = NEW.freundin,
    parterin = NEW.parterin
WHERE id = NEW.id;

GRANT SELECT ON page_4_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_4_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_4_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_4_delete_rule AS ON DELETE TO page_4_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_4_insert_rule AS ON INSERT TO page_4_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_5_view AS SELECT
    master_tbl.id AS id,
    master_tbl.schule_2 AS schule_2,
    master_tbl.klassenstufe AS klassenstufe,
    master_tbl.klassenwiederholung AS klassenwiederholung,
    master_tbl.abgang AS abgang,
    master_tbl.nicht_anwesende_tage AS nicht_anwesende_tage,
    master_tbl.unentschuldigte_tage AS unentschuldigte_tage,
    master_tbl.schulabschluss_1 AS schulabschluss_1,
    master_tbl.schulabschlusserwerbsland AS schulabschlusserwerbsland,
    master_tbl.schulabschlusserwerbsland_sonstige_nennung AS schulabschlusserwerbsland_sonstige_nennung
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_5_update_rule AS ON UPDATE TO page_5_view
DO INSTEAD UPDATE master_tbl SET
    schule_2 = NEW.schule_2,
    klassenstufe = NEW.klassenstufe,
    klassenwiederholung = NEW.klassenwiederholung,
    abgang = NEW.abgang,
    nicht_anwesende_tage = NEW.nicht_anwesende_tage,
    unentschuldigte_tage = NEW.unentschuldigte_tage,
    schulabschluss_1 = NEW.schulabschluss_1,
    schulabschlusserwerbsland = NEW.schulabschlusserwerbsland,
    schulabschlusserwerbsland_sonstige_nennung = NEW.schulabschlusserwerbsland_sonstige_nennung
WHERE id = NEW.id;

GRANT SELECT ON page_5_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_5_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_5_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_5_delete_rule AS ON DELETE TO page_5_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_5_insert_rule AS ON INSERT TO page_5_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_6_view AS SELECT
    master_tbl.id AS id,
    master_tbl.abschluss_betriebliche_ausbildung AS abschluss_betriebliche_ausbildung,
    master_tbl.abschluss_im_ausbildungsberuf AS abschluss_im_ausbildungsberuf,
    master_tbl.ausbildungsdauer AS ausbildungsdauer,
    master_tbl.wann_abgeschlossen AS wann_abgeschlossen,
    master_tbl.weitere_ausbildungen AS weitere_ausbildungen,
    master_tbl.weitere_ausbildungen_text AS weitere_ausbildungen_text,
    master_tbl.dauer_weitere_ausbildungen AS dauer_weitere_ausbildungen,
    master_tbl.abgebrochene_ausbildungen AS abgebrochene_ausbildungen,
    master_tbl.abgebrochene_ausbildungen_text AS abgebrochene_ausbildungen_text,
    master_tbl.abschluss_ausserbetriebliche_ausbildung AS abschluss_ausserbetriebliche_ausbildung,
    master_tbl.abschluss_im_ausbildungsberuf_1 AS abschluss_im_ausbildungsberuf_1,
    master_tbl.ausbildungsdauer_1 AS ausbildungsdauer_1,
    master_tbl.wann_abgeschlossen_1 AS wann_abgeschlossen_1,
    master_tbl.weitere_ausbildungen_1 AS weitere_ausbildungen_1,
    master_tbl.weitere_ausbildungen_text_1 AS weitere_ausbildungen_text_1,
    master_tbl.dauer_weitere_ausbildungen_1 AS dauer_weitere_ausbildungen_1,
    master_tbl.abgebrochene_ausbildungen_1 AS abgebrochene_ausbildungen_1,
    master_tbl.abgebrochene_ausbildungen_text_1 AS abgebrochene_ausbildungen_text_1,
    master_tbl.massnahmeteilnahme AS massnahmeteilnahme,
    master_tbl.letzte_massnahme AS letzte_massnahme,
    master_tbl.finanzierung AS finanzierung,
    master_tbl.austrittsdatum AS austrittsdatum,
    master_tbl.dauer AS dauer,
    master_tbl.weitere_massnahmen AS weitere_massnahmen,
    master_tbl.massnahme_1 AS massnahme_1,
    master_tbl.massnahme1_finanzierung AS massnahme1_finanzierung,
    master_tbl.massnahme_2 AS massnahme_2,
    master_tbl.massnahme2_finanzierung AS massnahme2_finanzierung,
    master_tbl.massnahme_3 AS massnahme_3,
    master_tbl.massnahme3_finanzierung AS massnahme3_finanzierung,
    master_tbl.abgebrochene_massnahmen AS abgebrochene_massnahmen,
    master_tbl.erwerbstaetigkeit_job AS erwerbstaetigkeit_job,
    master_tbl.letzte_taetigkeit AS letzte_taetigkeit,
    master_tbl.dauer_1 AS dauer_1,
    master_tbl.austrittsdatum_1 AS austrittsdatum_1,
    master_tbl.weitere_taetigkeiten AS weitere_taetigkeiten,
    master_tbl.weitere_jobs_text AS weitere_jobs_text,
    master_tbl.dauer_weitere_taetigkeiten AS dauer_weitere_taetigkeiten,
    master_tbl.arbeitslosigkeit_1 AS arbeitslosigkeit_1,
    master_tbl.anzahl_arbeitslosigkeit AS anzahl_arbeitslosigkeit,
    master_tbl.dauer_nach_letzter_beschaeftigung AS dauer_nach_letzter_beschaeftigung,
    master_tbl.dauer_insgesamt AS dauer_insgesamt,
    master_tbl.wehrpflicht_zivildienst_freiwilligenjahr_1 AS wehrpflicht_zivildienst_freiwilligenjahr_1,
    master_tbl.art_des_dienstes AS art_des_dienstes,
    master_tbl.dauer_2 AS dauer_2,
    master_tbl.mutterschutz_erziehungsurlaub AS mutterschutz_erziehungsurlaub,
    master_tbl.anzahl AS anzahl,
    master_tbl.dauer_letzter_erziehungsurlaub AS dauer_letzter_erziehungsurlaub,
    master_tbl.dauer_insgesamt_1 AS dauer_insgesamt_1
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_6_update_rule AS ON UPDATE TO page_6_view
DO INSTEAD UPDATE master_tbl SET
    abschluss_betriebliche_ausbildung = NEW.abschluss_betriebliche_ausbildung,
    abschluss_im_ausbildungsberuf = NEW.abschluss_im_ausbildungsberuf,
    ausbildungsdauer = NEW.ausbildungsdauer,
    wann_abgeschlossen = NEW.wann_abgeschlossen,
    weitere_ausbildungen = NEW.weitere_ausbildungen,
    weitere_ausbildungen_text = NEW.weitere_ausbildungen_text,
    dauer_weitere_ausbildungen = NEW.dauer_weitere_ausbildungen,
    abgebrochene_ausbildungen = NEW.abgebrochene_ausbildungen,
    abgebrochene_ausbildungen_text = NEW.abgebrochene_ausbildungen_text,
    abschluss_ausserbetriebliche_ausbildung = NEW.abschluss_ausserbetriebliche_ausbildung,
    abschluss_im_ausbildungsberuf_1 = NEW.abschluss_im_ausbildungsberuf_1,
    ausbildungsdauer_1 = NEW.ausbildungsdauer_1,
    wann_abgeschlossen_1 = NEW.wann_abgeschlossen_1,
    weitere_ausbildungen_1 = NEW.weitere_ausbildungen_1,
    weitere_ausbildungen_text_1 = NEW.weitere_ausbildungen_text_1,
    dauer_weitere_ausbildungen_1 = NEW.dauer_weitere_ausbildungen_1,
    abgebrochene_ausbildungen_1 = NEW.abgebrochene_ausbildungen_1,
    abgebrochene_ausbildungen_text_1 = NEW.abgebrochene_ausbildungen_text_1,
    massnahmeteilnahme = NEW.massnahmeteilnahme,
    letzte_massnahme = NEW.letzte_massnahme,
    finanzierung = NEW.finanzierung,
    austrittsdatum = NEW.austrittsdatum,
    dauer = NEW.dauer,
    weitere_massnahmen = NEW.weitere_massnahmen,
    massnahme_1 = NEW.massnahme_1,
    massnahme1_finanzierung = NEW.massnahme1_finanzierung,
    massnahme_2 = NEW.massnahme_2,
    massnahme2_finanzierung = NEW.massnahme2_finanzierung,
    massnahme_3 = NEW.massnahme_3,
    massnahme3_finanzierung = NEW.massnahme3_finanzierung,
    abgebrochene_massnahmen = NEW.abgebrochene_massnahmen,
    erwerbstaetigkeit_job = NEW.erwerbstaetigkeit_job,
    letzte_taetigkeit = NEW.letzte_taetigkeit,
    dauer_1 = NEW.dauer_1,
    austrittsdatum_1 = NEW.austrittsdatum_1,
    weitere_taetigkeiten = NEW.weitere_taetigkeiten,
    weitere_jobs_text = NEW.weitere_jobs_text,
    dauer_weitere_taetigkeiten = NEW.dauer_weitere_taetigkeiten,
    arbeitslosigkeit_1 = NEW.arbeitslosigkeit_1,
    anzahl_arbeitslosigkeit = NEW.anzahl_arbeitslosigkeit,
    dauer_nach_letzter_beschaeftigung = NEW.dauer_nach_letzter_beschaeftigung,
    dauer_insgesamt = NEW.dauer_insgesamt,
    wehrpflicht_zivildienst_freiwilligenjahr_1 = NEW.wehrpflicht_zivildienst_freiwilligenjahr_1,
    art_des_dienstes = NEW.art_des_dienstes,
    dauer_2 = NEW.dauer_2,
    mutterschutz_erziehungsurlaub = NEW.mutterschutz_erziehungsurlaub,
    anzahl = NEW.anzahl,
    dauer_letzter_erziehungsurlaub = NEW.dauer_letzter_erziehungsurlaub,
    dauer_insgesamt_1 = NEW.dauer_insgesamt_1
WHERE id = NEW.id;

GRANT SELECT ON page_6_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_6_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_6_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_6_delete_rule AS ON DELETE TO page_6_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_6_insert_rule AS ON INSERT TO page_6_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_7_view AS SELECT
    master_tbl.id AS id,
    master_tbl.kompetenzfeststellung AS kompetenzfeststellung,
    master_tbl.zielsetzung AS zielsetzung,
    master_tbl.begleitung AS begleitung,
    master_tbl.welche_vermittlung_nennung AS welche_vermittlung_nennung,
    master_tbl.freie_dokumentation AS freie_dokumentation
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_7_update_rule AS ON UPDATE TO page_7_view
DO INSTEAD UPDATE master_tbl SET
    kompetenzfeststellung = NEW.kompetenzfeststellung,
    zielsetzung = NEW.zielsetzung,
    begleitung = NEW.begleitung,
    welche_vermittlung_nennung = NEW.welche_vermittlung_nennung,
    freie_dokumentation = NEW.freie_dokumentation
WHERE id = NEW.id;

GRANT SELECT ON page_7_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_7_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_7_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_7_delete_rule AS ON DELETE TO page_7_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_7_insert_rule AS ON INSERT TO page_7_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_10_view AS SELECT
    master_tbl.id AS id,
    master_tbl.bildender_bereich AS bildender_bereich,
    master_tbl.bereich_berufliche_qualifizierung AS bereich_berufliche_qualifizierung,
    master_tbl.bereich_berufsvorbereitung AS bereich_berufsvorbereitung,
    master_tbl.bereich_lebensbewaeltigung AS bereich_lebensbewaeltigung,
    master_tbl.sonstiger_foerderbedarf AS sonstiger_foerderbedarf,
    master_tbl.sonstiger_foerderbedarf_nennung AS sonstiger_foerderbedarf_nennung,
    master_tbl.foerderzeitraum_start AS foerderzeitraum_start,
    master_tbl.foerderzeitraum_ende AS foerderzeitraum_ende
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_10_update_rule AS ON UPDATE TO page_10_view
DO INSTEAD UPDATE master_tbl SET
    bildender_bereich = NEW.bildender_bereich,
    bereich_berufliche_qualifizierung = NEW.bereich_berufliche_qualifizierung,
    bereich_berufsvorbereitung = NEW.bereich_berufsvorbereitung,
    bereich_lebensbewaeltigung = NEW.bereich_lebensbewaeltigung,
    sonstiger_foerderbedarf = NEW.sonstiger_foerderbedarf,
    sonstiger_foerderbedarf_nennung = NEW.sonstiger_foerderbedarf_nennung,
    foerderzeitraum_start = NEW.foerderzeitraum_start,
    foerderzeitraum_ende = NEW.foerderzeitraum_ende
WHERE id = NEW.id;

GRANT SELECT ON page_10_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_10_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_10_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_10_delete_rule AS ON DELETE TO page_10_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_10_insert_rule AS ON INSERT TO page_10_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_15_view AS SELECT
    master_tbl.id AS id,
    master_tbl.dokumentation_zum_foerderplan AS dokumentation_zum_foerderplan
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_15_update_rule AS ON UPDATE TO page_15_view
DO INSTEAD UPDATE master_tbl SET
    dokumentation_zum_foerderplan = NEW.dokumentation_zum_foerderplan
WHERE id = NEW.id;

GRANT SELECT ON page_15_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_15_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_15_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_15_delete_rule AS ON DELETE TO page_15_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_15_insert_rule AS ON INSERT TO page_15_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_16_view AS SELECT
    master_tbl.id AS id,
    master_tbl.anzahl_kontakte_termine AS anzahl_kontakte_termine,
    master_tbl.anzahl_der_stunden AS anzahl_der_stunden,
    master_tbl.zeitraum_fallmanagement AS zeitraum_fallmanagement,
    master_tbl.beendigung_5 AS beendigung_5,
    master_tbl.art_beendigung AS art_beendigung,
    master_tbl.abbruch_durch AS abbruch_durch,
    master_tbl.gruende_abbruch AS gruende_abbruch,
    master_tbl.gruende_sonstiges_nennung_4 AS gruende_sonstiges_nennung_4,
    master_tbl.vermittlung AS vermittlung,
    master_tbl.vermittlung_nennung AS vermittlung_nennung,
    master_tbl.veraenderungen_rechtskreis AS veraenderungen_rechtskreis
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_16_update_rule AS ON UPDATE TO page_16_view
DO INSTEAD UPDATE master_tbl SET
    anzahl_kontakte_termine = NEW.anzahl_kontakte_termine,
    anzahl_der_stunden = NEW.anzahl_der_stunden,
    zeitraum_fallmanagement = NEW.zeitraum_fallmanagement,
    beendigung_5 = NEW.beendigung_5,
    art_beendigung = NEW.art_beendigung,
    abbruch_durch = NEW.abbruch_durch,
    gruende_abbruch = NEW.gruende_abbruch,
    gruende_sonstiges_nennung_4 = NEW.gruende_sonstiges_nennung_4,
    vermittlung = NEW.vermittlung,
    vermittlung_nennung = NEW.vermittlung_nennung,
    veraenderungen_rechtskreis = NEW.veraenderungen_rechtskreis
WHERE id = NEW.id;

GRANT SELECT ON page_16_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_16_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_16_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_16_delete_rule AS ON DELETE TO page_16_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_16_insert_rule AS ON INSERT TO page_16_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_17_view AS SELECT
    master_tbl.id AS id,
    master_tbl.allgemein_bildender_schulabschluss AS allgemein_bildender_schulabschluss,
    master_tbl.allgemein_bildender_schulabschluss_sonstiges_nennung AS allgemein_bildender_schulabschluss_sonstiges_nennung,
    master_tbl.massnahmeabschluss AS massnahmeabschluss,
    master_tbl.schulart AS schulart,
    master_tbl.schule_sonstiges_nennung AS schule_sonstiges_nennung,
    master_tbl.klassenstufe_1 AS klassenstufe_1,
    master_tbl.ausbildung AS ausbildung,
    master_tbl.ausbildung_sonstiges_nennung AS ausbildung_sonstiges_nennung,
    master_tbl.umschulung AS umschulung,
    master_tbl.umschulung_nennung AS umschulung_nennung,
    master_tbl.arbeitstaetigkeit AS arbeitstaetigkeit,
    master_tbl.erwerbstatus AS erwerbstatus,
    master_tbl.erwerbsstatus_sonstiges_nennung AS erwerbsstatus_sonstiges_nennung,
    master_tbl.beschaeftigungsumfang AS beschaeftigungsumfang,
    master_tbl.kategorien_nichterwerbsarbeit_nichtausbildung AS kategorien_nichterwerbsarbeit_nichtausbildung,
    master_tbl.kategorien_sonstiges_nennung AS kategorien_sonstiges_nennung,
    master_tbl.zeitraum_nachbetreuung AS zeitraum_nachbetreuung,
    master_tbl.verbleib_nachbetreuung AS verbleib_nachbetreuung
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_17_update_rule AS ON UPDATE TO page_17_view
DO INSTEAD UPDATE master_tbl SET
    allgemein_bildender_schulabschluss = NEW.allgemein_bildender_schulabschluss,
    allgemein_bildender_schulabschluss_sonstiges_nennung = NEW.allgemein_bildender_schulabschluss_sonstiges_nennung,
    massnahmeabschluss = NEW.massnahmeabschluss,
    schulart = NEW.schulart,
    schule_sonstiges_nennung = NEW.schule_sonstiges_nennung,
    klassenstufe_1 = NEW.klassenstufe_1,
    ausbildung = NEW.ausbildung,
    ausbildung_sonstiges_nennung = NEW.ausbildung_sonstiges_nennung,
    umschulung = NEW.umschulung,
    umschulung_nennung = NEW.umschulung_nennung,
    arbeitstaetigkeit = NEW.arbeitstaetigkeit,
    erwerbstatus = NEW.erwerbstatus,
    erwerbsstatus_sonstiges_nennung = NEW.erwerbsstatus_sonstiges_nennung,
    beschaeftigungsumfang = NEW.beschaeftigungsumfang,
    kategorien_nichterwerbsarbeit_nichtausbildung = NEW.kategorien_nichterwerbsarbeit_nichtausbildung,
    kategorien_sonstiges_nennung = NEW.kategorien_sonstiges_nennung,
    zeitraum_nachbetreuung = NEW.zeitraum_nachbetreuung,
    verbleib_nachbetreuung = NEW.verbleib_nachbetreuung
WHERE id = NEW.id;

GRANT SELECT ON page_17_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_17_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_17_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_17_delete_rule AS ON DELETE TO page_17_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_17_insert_rule AS ON INSERT TO page_17_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_18_view AS SELECT
    master_tbl.id AS id,
    master_tbl.mitgliedschaft_organisationen_vereinen AS mitgliedschaft_organisationen_vereinen,
    master_tbl.intensitaet_mitgliedschaft AS intensitaet_mitgliedschaft,
    master_tbl.nutzung_kulturangeboten AS nutzung_kulturangeboten,
    master_tbl.haeufigkeit AS haeufigkeit,
    master_tbl.soziales_engagement AS soziales_engagement,
    master_tbl.unternehmungen_freunden AS unternehmungen_freunden,
    master_tbl.unternehmungen_familie AS unternehmungen_familie,
    master_tbl.peergroup AS peergroup,
    master_tbl.familie_1 AS familie_1,
    master_tbl.beziehung AS beziehung,
    master_tbl.arbeit_schule AS arbeit_schule,
    master_tbl.bester_freund_freundin AS bester_freund_freundin
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_18_update_rule AS ON UPDATE TO page_18_view
DO INSTEAD UPDATE master_tbl SET
    mitgliedschaft_organisationen_vereinen = NEW.mitgliedschaft_organisationen_vereinen,
    intensitaet_mitgliedschaft = NEW.intensitaet_mitgliedschaft,
    nutzung_kulturangeboten = NEW.nutzung_kulturangeboten,
    haeufigkeit = NEW.haeufigkeit,
    soziales_engagement = NEW.soziales_engagement,
    unternehmungen_freunden = NEW.unternehmungen_freunden,
    unternehmungen_familie = NEW.unternehmungen_familie,
    peergroup = NEW.peergroup,
    familie_1 = NEW.familie_1,
    beziehung = NEW.beziehung,
    arbeit_schule = NEW.arbeit_schule,
    bester_freund_freundin = NEW.bester_freund_freundin
WHERE id = NEW.id;

GRANT SELECT ON page_18_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_18_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_18_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_18_delete_rule AS ON DELETE TO page_18_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_18_insert_rule AS ON INSERT TO page_18_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_19_view AS SELECT
    master_tbl.id AS id,
    master_tbl.einschaetzung_zielerreichung AS einschaetzung_zielerreichung,
    master_tbl.erreichte_qualifikation AS erreichte_qualifikation,
    master_tbl.integration_ist_nachhaltig AS integration_ist_nachhaltig,
    master_tbl.selbststaendige_berufswegplanung AS selbststaendige_berufswegplanung,
    master_tbl.eigenstaendige_lebensgestaltung AS eigenstaendige_lebensgestaltung,
    master_tbl.soziale_bezugssysteme AS soziale_bezugssysteme,
    master_tbl.teilnahme_gesellschaftliches_leben AS teilnahme_gesellschaftliches_leben
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_19_update_rule AS ON UPDATE TO page_19_view
DO INSTEAD UPDATE master_tbl SET
    einschaetzung_zielerreichung = NEW.einschaetzung_zielerreichung,
    erreichte_qualifikation = NEW.erreichte_qualifikation,
    integration_ist_nachhaltig = NEW.integration_ist_nachhaltig,
    selbststaendige_berufswegplanung = NEW.selbststaendige_berufswegplanung,
    eigenstaendige_lebensgestaltung = NEW.eigenstaendige_lebensgestaltung,
    soziale_bezugssysteme = NEW.soziale_bezugssysteme,
    teilnahme_gesellschaftliches_leben = NEW.teilnahme_gesellschaftliches_leben
WHERE id = NEW.id;

GRANT SELECT ON page_19_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_19_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_19_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_19_delete_rule AS ON DELETE TO page_19_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_19_insert_rule AS ON INSERT TO page_19_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_20_view AS SELECT
    master_tbl.id AS id,
    master_tbl.freie_dokumentation_abschlussbewertung AS freie_dokumentation_abschlussbewertung
FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_20_update_rule AS ON UPDATE TO page_20_view
DO INSTEAD UPDATE master_tbl SET
    freie_dokumentation_abschlussbewertung = NEW.freie_dokumentation_abschlussbewertung
WHERE id = NEW.id;

GRANT SELECT ON page_20_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_20_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_20_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_20_delete_rule AS ON DELETE TO page_20_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_20_insert_rule AS ON INSERT TO page_20_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_8_view AS SELECT
    rg_kompetenzfestellung_tbl.id AS id,
    rg_kompetenzfestellung_tbl.verfahren_kompetenzfeststellung AS verfahren_kompetenzfeststellung,
    rg_kompetenzfestellung_tbl.externeranbieter AS externeranbieter,
    rg_kompetenzfestellung_tbl.assessment_center AS assessment_center,
    rg_kompetenzfestellung_tbl.interview AS interview,
    rg_kompetenzfestellung_tbl.biografieorientierte_methoden AS biografieorientierte_methoden,
    rg_kompetenzfestellung_tbl.selbsttest_selbstbeurteilung AS selbsttest_selbstbeurteilung,
    rg_kompetenzfestellung_tbl.sonstiges_2 AS sonstiges_2,
    rg_kompetenzfestellung_tbl.artdesverfahrens_nennung_sonstiges AS artdesverfahrens_nennung_sonstiges,
    rg_kompetenzfestellung_tbl.testverfahren AS testverfahren,
    rg_kompetenzfestellung_tbl.intelligenztest AS intelligenztest,
    rg_kompetenzfestellung_tbl.intelligenztest_nennung AS intelligenztest_nennung,
    rg_kompetenzfestellung_tbl.schultest AS schultest,
    rg_kompetenzfestellung_tbl.schultest_nennung AS schultest_nennung,
    rg_kompetenzfestellung_tbl.leistungstest AS leistungstest,
    rg_kompetenzfestellung_tbl.leistungstest_nennung AS leistungstest_nennung,
    rg_kompetenzfestellung_tbl.persoenlichkeitstest AS persoenlichkeitstest,
    rg_kompetenzfestellung_tbl.persoenlichkeitstest_nennung AS persoenlichkeitstest_nennung,
    rg_kompetenzfestellung_tbl.einstellungs_interessentest AS einstellungs_interessentest,
    rg_kompetenzfestellung_tbl.einstellungs_interessentest_nennung AS einstellungs_interessentest_nennung,
    rg_kompetenzfestellung_tbl.dauer_des_verfahrens AS dauer_des_verfahrens
FROM rg_kompetenzfestellung_tbl
    INNER JOIN master_tbl
    ON         rg_kompetenzfestellung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_8_update_rule AS ON UPDATE TO page_8_view
DO INSTEAD UPDATE rg_kompetenzfestellung_tbl SET
    verfahren_kompetenzfeststellung = NEW.verfahren_kompetenzfeststellung,
    externeranbieter = NEW.externeranbieter,
    assessment_center = NEW.assessment_center,
    interview = NEW.interview,
    biografieorientierte_methoden = NEW.biografieorientierte_methoden,
    selbsttest_selbstbeurteilung = NEW.selbsttest_selbstbeurteilung,
    sonstiges_2 = NEW.sonstiges_2,
    artdesverfahrens_nennung_sonstiges = NEW.artdesverfahrens_nennung_sonstiges,
    testverfahren = NEW.testverfahren,
    intelligenztest = NEW.intelligenztest,
    intelligenztest_nennung = NEW.intelligenztest_nennung,
    schultest = NEW.schultest,
    schultest_nennung = NEW.schultest_nennung,
    leistungstest = NEW.leistungstest,
    leistungstest_nennung = NEW.leistungstest_nennung,
    persoenlichkeitstest = NEW.persoenlichkeitstest,
    persoenlichkeitstest_nennung = NEW.persoenlichkeitstest_nennung,
    einstellungs_interessentest = NEW.einstellungs_interessentest,
    einstellungs_interessentest_nennung = NEW.einstellungs_interessentest_nennung,
    dauer_des_verfahrens = NEW.dauer_des_verfahrens
WHERE id = NEW.id;

GRANT SELECT ON page_8_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_8_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_8_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_8_delete_rule AS ON DELETE TO page_8_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_8_insert_rule AS ON INSERT TO page_8_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_9_view AS SELECT
    rg_kompetenzfestellung_tbl.id AS id,
    rg_kompetenzfestellung_tbl.rechenfaehigkeit AS rechenfaehigkeit,
    rg_kompetenzfestellung_tbl.raeumliches_vorstellungsvermoegen AS raeumliches_vorstellungsvermoegen,
    rg_kompetenzfestellung_tbl.grundverstaendnis_naturwissenschaftlicher_zusammenhaenge AS grundverstaendnis_naturwissenschaftlicher_zusammenhaenge,
    rg_kompetenzfestellung_tbl.logisches_denken AS logisches_denken,
    rg_kompetenzfestellung_tbl.wirtschaftliche_grundkenntnisse AS wirtschaftliche_grundkenntnisse,
    rg_kompetenzfestellung_tbl.allgemeine_pc_kenntnisse AS allgemeine_pc_kenntnisse,
    rg_kompetenzfestellung_tbl.spezielle_pc_kenntnisse_grafikbearbeitung_musikbearbeitung AS spezielle_pc_kenntnisse_grafikbearbeitung_musikbearbeitung,
    rg_kompetenzfestellung_tbl.internet_anwendungen_und_e_mail_kenntnisse AS internet_anwendungen_und_e_mail_kenntnisse,
    rg_kompetenzfestellung_tbl.andere_medien AS andere_medien,
    rg_kompetenzfestellung_tbl.hand_fingergeschick AS hand_fingergeschick,
    rg_kompetenzfestellung_tbl.formauffassung AS formauffassung,
    rg_kompetenzfestellung_tbl.umgang_maschinen_werkzeugen AS umgang_maschinen_werkzeugen,
    rg_kompetenzfestellung_tbl.gestalterische_fantasie AS gestalterische_fantasie
FROM rg_kompetenzfestellung_tbl
    INNER JOIN master_tbl
    ON         rg_kompetenzfestellung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_9_update_rule AS ON UPDATE TO page_9_view
DO INSTEAD UPDATE rg_kompetenzfestellung_tbl SET
    rechenfaehigkeit = NEW.rechenfaehigkeit,
    raeumliches_vorstellungsvermoegen = NEW.raeumliches_vorstellungsvermoegen,
    grundverstaendnis_naturwissenschaftlicher_zusammenhaenge = NEW.grundverstaendnis_naturwissenschaftlicher_zusammenhaenge,
    logisches_denken = NEW.logisches_denken,
    wirtschaftliche_grundkenntnisse = NEW.wirtschaftliche_grundkenntnisse,
    allgemeine_pc_kenntnisse = NEW.allgemeine_pc_kenntnisse,
    spezielle_pc_kenntnisse_grafikbearbeitung_musikbearbeitung = NEW.spezielle_pc_kenntnisse_grafikbearbeitung_musikbearbeitung,
    internet_anwendungen_und_e_mail_kenntnisse = NEW.internet_anwendungen_und_e_mail_kenntnisse,
    andere_medien = NEW.andere_medien,
    hand_fingergeschick = NEW.hand_fingergeschick,
    formauffassung = NEW.formauffassung,
    umgang_maschinen_werkzeugen = NEW.umgang_maschinen_werkzeugen,
    gestalterische_fantasie = NEW.gestalterische_fantasie
WHERE id = NEW.id;

GRANT SELECT ON page_9_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_9_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_9_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_9_delete_rule AS ON DELETE TO page_9_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_9_insert_rule AS ON INSERT TO page_9_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_26_view AS SELECT
    rg_kompetenzfestellung_tbl.id AS id,
    rg_kompetenzfestellung_tbl.methoden_der_informationsgewinnung AS methoden_der_informationsgewinnung,
    rg_kompetenzfestellung_tbl.methoden_der_informationsverarbeitung AS methoden_der_informationsverarbeitung,
    rg_kompetenzfestellung_tbl.einsatz_lern_und_arbeitstechniken AS einsatz_lern_und_arbeitstechniken,
    rg_kompetenzfestellung_tbl.problemloesungsorientierung AS problemloesungsorientierung
FROM rg_kompetenzfestellung_tbl
    INNER JOIN master_tbl
    ON         rg_kompetenzfestellung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_26_update_rule AS ON UPDATE TO page_26_view
DO INSTEAD UPDATE rg_kompetenzfestellung_tbl SET
    methoden_der_informationsgewinnung = NEW.methoden_der_informationsgewinnung,
    methoden_der_informationsverarbeitung = NEW.methoden_der_informationsverarbeitung,
    einsatz_lern_und_arbeitstechniken = NEW.einsatz_lern_und_arbeitstechniken,
    problemloesungsorientierung = NEW.problemloesungsorientierung
WHERE id = NEW.id;

GRANT SELECT ON page_26_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_26_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_26_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_26_delete_rule AS ON DELETE TO page_26_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_26_insert_rule AS ON INSERT TO page_26_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_27_view AS SELECT
    rg_kompetenzfestellung_tbl.id AS id,
    rg_kompetenzfestellung_tbl.texte_lesen_und_verstehen AS texte_lesen_und_verstehen,
    rg_kompetenzfestellung_tbl.texte_schreiben_und_formulieren AS texte_schreiben_und_formulieren,
    rg_kompetenzfestellung_tbl.deutsche_sprache AS deutsche_sprache,
    rg_kompetenzfestellung_tbl.sprache1 AS sprache1,
    rg_kompetenzfestellung_tbl.sprache1_sonstige_nennung AS sprache1_sonstige_nennung,
    rg_kompetenzfestellung_tbl.beherrschung_sprache1 AS beherrschung_sprache1,
    rg_kompetenzfestellung_tbl.sprache2 AS sprache2,
    rg_kompetenzfestellung_tbl.sprache2_sonstige_nennung AS sprache2_sonstige_nennung,
    rg_kompetenzfestellung_tbl.beherrschung_sprache2 AS beherrschung_sprache2,
    rg_kompetenzfestellung_tbl.sprache3 AS sprache3,
    rg_kompetenzfestellung_tbl.sprache3_sonstige_nennung AS sprache3_sonstige_nennung,
    rg_kompetenzfestellung_tbl.beherrschung_sprache3 AS beherrschung_sprache3
FROM rg_kompetenzfestellung_tbl
    INNER JOIN master_tbl
    ON         rg_kompetenzfestellung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_27_update_rule AS ON UPDATE TO page_27_view
DO INSTEAD UPDATE rg_kompetenzfestellung_tbl SET
    texte_lesen_und_verstehen = NEW.texte_lesen_und_verstehen,
    texte_schreiben_und_formulieren = NEW.texte_schreiben_und_formulieren,
    deutsche_sprache = NEW.deutsche_sprache,
    sprache1 = NEW.sprache1,
    sprache1_sonstige_nennung = NEW.sprache1_sonstige_nennung,
    beherrschung_sprache1 = NEW.beherrschung_sprache1,
    sprache2 = NEW.sprache2,
    sprache2_sonstige_nennung = NEW.sprache2_sonstige_nennung,
    beherrschung_sprache2 = NEW.beherrschung_sprache2,
    sprache3 = NEW.sprache3,
    sprache3_sonstige_nennung = NEW.sprache3_sonstige_nennung,
    beherrschung_sprache3 = NEW.beherrschung_sprache3
WHERE id = NEW.id;

GRANT SELECT ON page_27_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_27_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_27_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_27_delete_rule AS ON DELETE TO page_27_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_27_insert_rule AS ON INSERT TO page_27_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_28_view AS SELECT
    rg_kompetenzfestellung_tbl.id AS id,
    rg_kompetenzfestellung_tbl.kommunikationsfaehigkeit AS kommunikationsfaehigkeit,
    rg_kompetenzfestellung_tbl.kritikfaehigkeit AS kritikfaehigkeit,
    rg_kompetenzfestellung_tbl.konfliktfaehigkeit AS konfliktfaehigkeit,
    rg_kompetenzfestellung_tbl.kreativitaet AS kreativitaet,
    rg_kompetenzfestellung_tbl.team_und_kooperationsfaehigkeit AS team_und_kooperationsfaehigkeit,
    rg_kompetenzfestellung_tbl.solidarisches_verhalten AS solidarisches_verhalten,
    rg_kompetenzfestellung_tbl.selbstaendiges_lernen_arbeiten AS selbstaendiges_lernen_arbeiten,
    rg_kompetenzfestellung_tbl.selbstreflexion_selbsteinschaetzung AS selbstreflexion_selbsteinschaetzung,
    rg_kompetenzfestellung_tbl.verantwortungsuebernahme AS verantwortungsuebernahme,
    rg_kompetenzfestellung_tbl.zielorientierung AS zielorientierung,
    rg_kompetenzfestellung_tbl.zuverlaessigkeit AS zuverlaessigkeit,
    rg_kompetenzfestellung_tbl.altersgerechter_entwicklungsstand AS altersgerechter_entwicklungsstand,
    rg_kompetenzfestellung_tbl.gesundheitliche_voraussetzungen AS gesundheitliche_voraussetzungen,
    rg_kompetenzfestellung_tbl.konzentration AS konzentration,
    rg_kompetenzfestellung_tbl.ausdauer AS ausdauer,
    rg_kompetenzfestellung_tbl.durchhaltevermoegen AS durchhaltevermoegen,
    rg_kompetenzfestellung_tbl.merkfaehigkeit AS merkfaehigkeit,
    rg_kompetenzfestellung_tbl.bearbeitungsgeschwindigkeit AS bearbeitungsgeschwindigkeit,
    rg_kompetenzfestellung_tbl.befaehigung_zu_daueraufmerksamkeit AS befaehigung_zu_daueraufmerksamkeit,
    rg_kompetenzfestellung_tbl.leistungsbereitschaft AS leistungsbereitschaft,
    rg_kompetenzfestellung_tbl.sorgfalt AS sorgfalt,
    rg_kompetenzfestellung_tbl.umgangsformen AS umgangsformen
FROM rg_kompetenzfestellung_tbl
    INNER JOIN master_tbl
    ON         rg_kompetenzfestellung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_28_update_rule AS ON UPDATE TO page_28_view
DO INSTEAD UPDATE rg_kompetenzfestellung_tbl SET
    kommunikationsfaehigkeit = NEW.kommunikationsfaehigkeit,
    kritikfaehigkeit = NEW.kritikfaehigkeit,
    konfliktfaehigkeit = NEW.konfliktfaehigkeit,
    kreativitaet = NEW.kreativitaet,
    team_und_kooperationsfaehigkeit = NEW.team_und_kooperationsfaehigkeit,
    solidarisches_verhalten = NEW.solidarisches_verhalten,
    selbstaendiges_lernen_arbeiten = NEW.selbstaendiges_lernen_arbeiten,
    selbstreflexion_selbsteinschaetzung = NEW.selbstreflexion_selbsteinschaetzung,
    verantwortungsuebernahme = NEW.verantwortungsuebernahme,
    zielorientierung = NEW.zielorientierung,
    zuverlaessigkeit = NEW.zuverlaessigkeit,
    altersgerechter_entwicklungsstand = NEW.altersgerechter_entwicklungsstand,
    gesundheitliche_voraussetzungen = NEW.gesundheitliche_voraussetzungen,
    konzentration = NEW.konzentration,
    ausdauer = NEW.ausdauer,
    durchhaltevermoegen = NEW.durchhaltevermoegen,
    merkfaehigkeit = NEW.merkfaehigkeit,
    bearbeitungsgeschwindigkeit = NEW.bearbeitungsgeschwindigkeit,
    befaehigung_zu_daueraufmerksamkeit = NEW.befaehigung_zu_daueraufmerksamkeit,
    leistungsbereitschaft = NEW.leistungsbereitschaft,
    sorgfalt = NEW.sorgfalt,
    umgangsformen = NEW.umgangsformen
WHERE id = NEW.id;

GRANT SELECT ON page_28_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_28_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_28_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_28_delete_rule AS ON DELETE TO page_28_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_28_insert_rule AS ON INSERT TO page_28_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_29_view AS SELECT
    rg_kompetenzfestellung_tbl.id AS id,
    rg_kompetenzfestellung_tbl.dokumentation_kompetenzfeststellung AS dokumentation_kompetenzfeststellung
FROM rg_kompetenzfestellung_tbl
    INNER JOIN master_tbl
    ON         rg_kompetenzfestellung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_29_update_rule AS ON UPDATE TO page_29_view
DO INSTEAD UPDATE rg_kompetenzfestellung_tbl SET
    dokumentation_kompetenzfeststellung = NEW.dokumentation_kompetenzfeststellung
WHERE id = NEW.id;

GRANT SELECT ON page_29_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_29_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_29_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_29_delete_rule AS ON DELETE TO page_29_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_29_insert_rule AS ON INSERT TO page_29_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_21_view AS SELECT
    rg_angebote_bildenden_bereich_tbl.id AS id,
    rg_angebote_bildenden_bereich_tbl.angebotsform AS angebotsform,
    rg_angebote_bildenden_bereich_tbl.art_der_unterstuetzung AS art_der_unterstuetzung,
    rg_angebote_bildenden_bereich_tbl.durchfuehrende_institution AS durchfuehrende_institution,
    rg_angebote_bildenden_bereich_tbl.unterstuetzung_start AS unterstuetzung_start,
    rg_angebote_bildenden_bereich_tbl.unterstuetzung_ende AS unterstuetzung_ende,
    rg_angebote_bildenden_bereich_tbl.zeitlicher_umfang_unterstuetzung AS zeitlicher_umfang_unterstuetzung,
    rg_angebote_bildenden_bereich_tbl.beendigung_1 AS beendigung_1,
    rg_angebote_bildenden_bereich_tbl.beendigung_durch AS beendigung_durch,
    rg_angebote_bildenden_bereich_tbl.gruende_vorzeitige_beendigung AS gruende_vorzeitige_beendigung,
    rg_angebote_bildenden_bereich_tbl.gruende_sonstiges_nennung AS gruende_sonstiges_nennung,
    rg_angebote_bildenden_bereich_tbl.nachweis_bescheinigung AS nachweis_bescheinigung,
    rg_angebote_bildenden_bereich_tbl.sonstiges_nachweis_bescheinigung AS sonstiges_nachweis_bescheinigung
FROM rg_angebote_bildenden_bereich_tbl
    INNER JOIN master_tbl
    ON         rg_angebote_bildenden_bereich_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_21_update_rule AS ON UPDATE TO page_21_view
DO INSTEAD UPDATE rg_angebote_bildenden_bereich_tbl SET
    angebotsform = NEW.angebotsform,
    art_der_unterstuetzung = NEW.art_der_unterstuetzung,
    durchfuehrende_institution = NEW.durchfuehrende_institution,
    unterstuetzung_start = NEW.unterstuetzung_start,
    unterstuetzung_ende = NEW.unterstuetzung_ende,
    zeitlicher_umfang_unterstuetzung = NEW.zeitlicher_umfang_unterstuetzung,
    beendigung_1 = NEW.beendigung_1,
    beendigung_durch = NEW.beendigung_durch,
    gruende_vorzeitige_beendigung = NEW.gruende_vorzeitige_beendigung,
    gruende_sonstiges_nennung = NEW.gruende_sonstiges_nennung,
    nachweis_bescheinigung = NEW.nachweis_bescheinigung,
    sonstiges_nachweis_bescheinigung = NEW.sonstiges_nachweis_bescheinigung
WHERE id = NEW.id;

GRANT SELECT ON page_21_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_21_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_21_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_21_delete_rule AS ON DELETE TO page_21_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_21_insert_rule AS ON INSERT TO page_21_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_22_view AS SELECT
    rg_angebote_berufsvorbereitung_tbl.id AS id,
    rg_angebote_berufsvorbereitung_tbl.angebotsform_1 AS angebotsform_1,
    rg_angebote_berufsvorbereitung_tbl.art_der_unterstuetzung_1 AS art_der_unterstuetzung_1,
    rg_angebote_berufsvorbereitung_tbl.unterstuetzung_sonstiges_nennung AS unterstuetzung_sonstiges_nennung,
    rg_angebote_berufsvorbereitung_tbl.durchfuehrende_institution_1 AS durchfuehrende_institution_1,
    rg_angebote_berufsvorbereitung_tbl.unterstuetzung_start_1 AS unterstuetzung_start_1,
    rg_angebote_berufsvorbereitung_tbl.unterstuetzung_ende_1 AS unterstuetzung_ende_1,
    rg_angebote_berufsvorbereitung_tbl.zeitlicher_umfang_unterstuetzung_1 AS zeitlicher_umfang_unterstuetzung_1,
    rg_angebote_berufsvorbereitung_tbl.beendigung_2 AS beendigung_2,
    rg_angebote_berufsvorbereitung_tbl.beendigung_durch_1 AS beendigung_durch_1,
    rg_angebote_berufsvorbereitung_tbl.gruende_vorzeitige_beendigung_1 AS gruende_vorzeitige_beendigung_1,
    rg_angebote_berufsvorbereitung_tbl.gruende_sonstiges_nennung_1 AS gruende_sonstiges_nennung_1,
    rg_angebote_berufsvorbereitung_tbl.nachweis_bescheinigung_1 AS nachweis_bescheinigung_1,
    rg_angebote_berufsvorbereitung_tbl.sonstiges_nachweis_bescheinigung_1 AS sonstiges_nachweis_bescheinigung_1
FROM rg_angebote_berufsvorbereitung_tbl
    INNER JOIN master_tbl
    ON         rg_angebote_berufsvorbereitung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_22_update_rule AS ON UPDATE TO page_22_view
DO INSTEAD UPDATE rg_angebote_berufsvorbereitung_tbl SET
    angebotsform_1 = NEW.angebotsform_1,
    art_der_unterstuetzung_1 = NEW.art_der_unterstuetzung_1,
    unterstuetzung_sonstiges_nennung = NEW.unterstuetzung_sonstiges_nennung,
    durchfuehrende_institution_1 = NEW.durchfuehrende_institution_1,
    unterstuetzung_start_1 = NEW.unterstuetzung_start_1,
    unterstuetzung_ende_1 = NEW.unterstuetzung_ende_1,
    zeitlicher_umfang_unterstuetzung_1 = NEW.zeitlicher_umfang_unterstuetzung_1,
    beendigung_2 = NEW.beendigung_2,
    beendigung_durch_1 = NEW.beendigung_durch_1,
    gruende_vorzeitige_beendigung_1 = NEW.gruende_vorzeitige_beendigung_1,
    gruende_sonstiges_nennung_1 = NEW.gruende_sonstiges_nennung_1,
    nachweis_bescheinigung_1 = NEW.nachweis_bescheinigung_1,
    sonstiges_nachweis_bescheinigung_1 = NEW.sonstiges_nachweis_bescheinigung_1
WHERE id = NEW.id;

GRANT SELECT ON page_22_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_22_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_22_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_22_delete_rule AS ON DELETE TO page_22_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_22_insert_rule AS ON INSERT TO page_22_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_23_view AS SELECT
    rg_angebote_berufliche_qualifizierung_tbl.id AS id,
    rg_angebote_berufliche_qualifizierung_tbl.angebotsform_2 AS angebotsform_2,
    rg_angebote_berufliche_qualifizierung_tbl.angebote_im_bereich AS angebote_im_bereich,
    rg_angebote_berufliche_qualifizierung_tbl.unterstuetzung_sonstiges_nennung_1 AS unterstuetzung_sonstiges_nennung_1,
    rg_angebote_berufliche_qualifizierung_tbl.durchfuehrende_institution_2 AS durchfuehrende_institution_2,
    rg_angebote_berufliche_qualifizierung_tbl.unterstuetzung_start_2 AS unterstuetzung_start_2,
    rg_angebote_berufliche_qualifizierung_tbl.unterstuetzung_ende_2 AS unterstuetzung_ende_2,
    rg_angebote_berufliche_qualifizierung_tbl.zeitlicher_umfang_unterstuetzung_2 AS zeitlicher_umfang_unterstuetzung_2,
    rg_angebote_berufliche_qualifizierung_tbl.beendigung_3 AS beendigung_3,
    rg_angebote_berufliche_qualifizierung_tbl.beendigung_durch_2 AS beendigung_durch_2,
    rg_angebote_berufliche_qualifizierung_tbl.gruende_vorzeitige_beendigung_2 AS gruende_vorzeitige_beendigung_2,
    rg_angebote_berufliche_qualifizierung_tbl.gruende_sonstiges_nennung_2 AS gruende_sonstiges_nennung_2,
    rg_angebote_berufliche_qualifizierung_tbl.nachweis_bescheinigung_2 AS nachweis_bescheinigung_2,
    rg_angebote_berufliche_qualifizierung_tbl.sonstiges_nachweis_bescheinigung_2 AS sonstiges_nachweis_bescheinigung_2
FROM rg_angebote_berufliche_qualifizierung_tbl
    INNER JOIN master_tbl
    ON         rg_angebote_berufliche_qualifizierung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_23_update_rule AS ON UPDATE TO page_23_view
DO INSTEAD UPDATE rg_angebote_berufliche_qualifizierung_tbl SET
    angebotsform_2 = NEW.angebotsform_2,
    angebote_im_bereich = NEW.angebote_im_bereich,
    unterstuetzung_sonstiges_nennung_1 = NEW.unterstuetzung_sonstiges_nennung_1,
    durchfuehrende_institution_2 = NEW.durchfuehrende_institution_2,
    unterstuetzung_start_2 = NEW.unterstuetzung_start_2,
    unterstuetzung_ende_2 = NEW.unterstuetzung_ende_2,
    zeitlicher_umfang_unterstuetzung_2 = NEW.zeitlicher_umfang_unterstuetzung_2,
    beendigung_3 = NEW.beendigung_3,
    beendigung_durch_2 = NEW.beendigung_durch_2,
    gruende_vorzeitige_beendigung_2 = NEW.gruende_vorzeitige_beendigung_2,
    gruende_sonstiges_nennung_2 = NEW.gruende_sonstiges_nennung_2,
    nachweis_bescheinigung_2 = NEW.nachweis_bescheinigung_2,
    sonstiges_nachweis_bescheinigung_2 = NEW.sonstiges_nachweis_bescheinigung_2
WHERE id = NEW.id;

GRANT SELECT ON page_23_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_23_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_23_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_23_delete_rule AS ON DELETE TO page_23_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_23_insert_rule AS ON INSERT TO page_23_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_24_view AS SELECT
    rg_angebote_lebensbewaeltigung_tbl.id AS id,
    rg_angebote_lebensbewaeltigung_tbl.angebotsform_3 AS angebotsform_3,
    rg_angebote_lebensbewaeltigung_tbl.unterstuetzung_angebotsform_sonstiges_nennung AS unterstuetzung_angebotsform_sonstiges_nennung,
    rg_angebote_lebensbewaeltigung_tbl.art_der_unterstuetzung_2 AS art_der_unterstuetzung_2,
    rg_angebote_lebensbewaeltigung_tbl.unterstuetzung_sonstiges_nennung_2 AS unterstuetzung_sonstiges_nennung_2,
    rg_angebote_lebensbewaeltigung_tbl.durchfuehrende_institution_3 AS durchfuehrende_institution_3,
    rg_angebote_lebensbewaeltigung_tbl.unterstuetzung_start_3 AS unterstuetzung_start_3,
    rg_angebote_lebensbewaeltigung_tbl.unterstuetzung_ende_3 AS unterstuetzung_ende_3,
    rg_angebote_lebensbewaeltigung_tbl.zeitlicher_umfang_unterstuetzung_3 AS zeitlicher_umfang_unterstuetzung_3,
    rg_angebote_lebensbewaeltigung_tbl.beendigung_4 AS beendigung_4,
    rg_angebote_lebensbewaeltigung_tbl.beendigung_durch_3 AS beendigung_durch_3,
    rg_angebote_lebensbewaeltigung_tbl.gruende_vorzeitige_beendigung_3 AS gruende_vorzeitige_beendigung_3,
    rg_angebote_lebensbewaeltigung_tbl.gruende_sonstiges_nennung_3 AS gruende_sonstiges_nennung_3,
    rg_angebote_lebensbewaeltigung_tbl.nachweis_bescheinigung_3 AS nachweis_bescheinigung_3,
    rg_angebote_lebensbewaeltigung_tbl.sonstiges_nachweis_bescheinigung_3 AS sonstiges_nachweis_bescheinigung_3
FROM rg_angebote_lebensbewaeltigung_tbl
    INNER JOIN master_tbl
    ON         rg_angebote_lebensbewaeltigung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE page_24_update_rule AS ON UPDATE TO page_24_view
DO INSTEAD UPDATE rg_angebote_lebensbewaeltigung_tbl SET
    angebotsform_3 = NEW.angebotsform_3,
    unterstuetzung_angebotsform_sonstiges_nennung = NEW.unterstuetzung_angebotsform_sonstiges_nennung,
    art_der_unterstuetzung_2 = NEW.art_der_unterstuetzung_2,
    unterstuetzung_sonstiges_nennung_2 = NEW.unterstuetzung_sonstiges_nennung_2,
    durchfuehrende_institution_3 = NEW.durchfuehrende_institution_3,
    unterstuetzung_start_3 = NEW.unterstuetzung_start_3,
    unterstuetzung_ende_3 = NEW.unterstuetzung_ende_3,
    zeitlicher_umfang_unterstuetzung_3 = NEW.zeitlicher_umfang_unterstuetzung_3,
    beendigung_4 = NEW.beendigung_4,
    beendigung_durch_3 = NEW.beendigung_durch_3,
    gruende_vorzeitige_beendigung_3 = NEW.gruende_vorzeitige_beendigung_3,
    gruende_sonstiges_nennung_3 = NEW.gruende_sonstiges_nennung_3,
    nachweis_bescheinigung_3 = NEW.nachweis_bescheinigung_3,
    sonstiges_nachweis_bescheinigung_3 = NEW.sonstiges_nachweis_bescheinigung_3
WHERE id = NEW.id;

GRANT SELECT ON page_24_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_24_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_24_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_24_delete_rule AS ON DELETE TO page_24_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_24_insert_rule AS ON INSERT TO page_24_view
    DO INSTEAD NOTHING;

--
--
-- views for master table and repeat groups
--
CREATE OR REPLACE VIEW master_tbl_view
AS SELECT master_tbl.* FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE master_tbl_update_rule AS ON UPDATE TO master_tbl_view
DO INSTEAD UPDATE master_tbl SET
    einverstaendniserklaerung = NEW.einverstaendniserklaerung,
    fkz = NEW.fkz,
    name_kompetenzagentur = NEW.name_kompetenzagentur,
    vorname = NEW.vorname,
    name = NEW.name,
    erstgespraech = NEW.erstgespraech,
    zugang_kompetenzagentur = NEW.zugang_kompetenzagentur,
    kundennummer = NEW.kundennummer,
    schulpflichtig = NEW.schulpflichtig,
    noch_schueler = NEW.noch_schueler,
    bei_eintritt_ist_der_jugendliche = NEW.bei_eintritt_ist_der_jugendliche,
    rechtlicher_kontext = NEW.rechtlicher_kontext,
    schnittstelle_zu = NEW.schnittstelle_zu,
    kontaktdaten_arge = NEW.kontaktdaten_arge,
    kundennummer_arge = NEW.kundennummer_arge,
    kontaktdaten_agentur_fuer_arbeit = NEW.kontaktdaten_agentur_fuer_arbeit,
    kundennummer_agentur_fuer_arbeit = NEW.kundennummer_agentur_fuer_arbeit,
    kundennummer_jugendamt = NEW.kundennummer_jugendamt,
    kontaktdaten_jugendamt = NEW.kontaktdaten_jugendamt,
    kontaktdaten_schule = NEW.kontaktdaten_schule,
    schulname = NEW.schulname,
    kontaktdaten_gerichtshilfe = NEW.kontaktdaten_gerichtshilfe,
    kundennummer_gerichtshilfe = NEW.kundennummer_gerichtshilfe,
    kontaktdaten_weitere_institutionen = NEW.kontaktdaten_weitere_institutionen,
    vorname_1 = NEW.vorname_1,
    name_1 = NEW.name_1,
    geschlecht = NEW.geschlecht,
    geburtsdatum = NEW.geburtsdatum,
    familienstand = NEW.familienstand,
    strasse = NEW.strasse,
    plz = NEW.plz,
    wohnort = NEW.wohnort,
    groesse_wohnort = NEW.groesse_wohnort,
    vorwahl = NEW.vorwahl,
    telefon_1 = NEW.telefon_1,
    mobiltelefon = NEW.mobiltelefon,
    email = NEW.email,
    telefon_sonstige = NEW.telefon_sonstige,
    zweiter_wohnort = NEW.zweiter_wohnort,
    geburtsland = NEW.geburtsland,
    geburtsland_sonstige_nennung = NEW.geburtsland_sonstige_nennung,
    muttersprache = NEW.muttersprache,
    muttersprache_sonstige_nennung = NEW.muttersprache_sonstige_nennung,
    staatsangehoerigkeit = NEW.staatsangehoerigkeit,
    staatsangehoerigkeit_sonstige_nennung = NEW.staatsangehoerigkeit_sonstige_nennung,
    aufenthaltsstatus = NEW.aufenthaltsstatus,
    aufenthaltsdauer = NEW.aufenthaltsdauer,
    vater = NEW.vater,
    mutter = NEW.mutter,
    familiaere_situation_1 = NEW.familiaere_situation_1,
    vorname_2 = NEW.vorname_2,
    name_2 = NEW.name_2,
    vorwahl_1 = NEW.vorwahl_1,
    telefon_1_1 = NEW.telefon_1_1,
    mobiltelefon_1 = NEW.mobiltelefon_1,
    email_1 = NEW.email_1,
    geschwister = NEW.geschwister,
    anzahl_kinder = NEW.anzahl_kinder,
    haushalt_jugendlicher = NEW.haushalt_jugendlicher,
    alleinerziehend = NEW.alleinerziehend,
    familie_selbststaendig = NEW.familie_selbststaendig,
    familie_staatliche_unterstuetzung = NEW.familie_staatliche_unterstuetzung,
    jugendlicher_selbststaendig = NEW.jugendlicher_selbststaendig,
    jugendlicher_staatliche_unterstuetzung = NEW.jugendlicher_staatliche_unterstuetzung,
    jugendlicher_unterstuetzung_soziales_umfeld = NEW.jugendlicher_unterstuetzung_soziales_umfeld,
    wohnaufenthalt = NEW.wohnaufenthalt,
    arzt_gutachten_physisch = NEW.arzt_gutachten_physisch,
    physische_beeintraechtigungen = NEW.physische_beeintraechtigungen,
    physische_beeintraechtigungen_text = NEW.physische_beeintraechtigungen_text,
    arzt_gutachten_psychisch = NEW.arzt_gutachten_psychisch,
    psychische_beeintraechtigungen = NEW.psychische_beeintraechtigungen,
    psychische_beeintraechtigungen_text = NEW.psychische_beeintraechtigungen_text,
    mutter_2 = NEW.mutter_2,
    vater_2 = NEW.vater_2,
    grosseltern = NEW.grosseltern,
    geschwister_2 = NEW.geschwister_2,
    pflegefamilie_1 = NEW.pflegefamilie_1,
    stiefmutter = NEW.stiefmutter,
    stiefvater = NEW.stiefvater,
    verwandte_1 = NEW.verwandte_1,
    partner = NEW.partner,
    heimbetreuerin = NEW.heimbetreuerin,
    betreuerin = NEW.betreuerin,
    lehrerin = NEW.lehrerin,
    mitbewohner = NEW.mitbewohner,
    partner_mutter = NEW.partner_mutter,
    partner_vater = NEW.partner_vater,
    freund = NEW.freund,
    keine_bezugsperson = NEW.keine_bezugsperson,
    hobbies_1 = NEW.hobbies_1,
    hobbies_text = NEW.hobbies_text,
    mitarbeit_organisationen = NEW.mitarbeit_organisationen,
    organisationen_vereine_text = NEW.organisationen_vereine_text,
    sonstiges_1 = NEW.sonstiges_1,
    sonstiges_text = NEW.sonstiges_text,
    clique = NEW.clique,
    freundin = NEW.freundin,
    parterin = NEW.parterin,
    schule_2 = NEW.schule_2,
    klassenstufe = NEW.klassenstufe,
    klassenwiederholung = NEW.klassenwiederholung,
    abgang = NEW.abgang,
    nicht_anwesende_tage = NEW.nicht_anwesende_tage,
    unentschuldigte_tage = NEW.unentschuldigte_tage,
    schulabschluss_1 = NEW.schulabschluss_1,
    schulabschlusserwerbsland = NEW.schulabschlusserwerbsland,
    schulabschlusserwerbsland_sonstige_nennung = NEW.schulabschlusserwerbsland_sonstige_nennung,
    abschluss_betriebliche_ausbildung = NEW.abschluss_betriebliche_ausbildung,
    abschluss_im_ausbildungsberuf = NEW.abschluss_im_ausbildungsberuf,
    ausbildungsdauer = NEW.ausbildungsdauer,
    wann_abgeschlossen = NEW.wann_abgeschlossen,
    weitere_ausbildungen = NEW.weitere_ausbildungen,
    weitere_ausbildungen_text = NEW.weitere_ausbildungen_text,
    dauer_weitere_ausbildungen = NEW.dauer_weitere_ausbildungen,
    abgebrochene_ausbildungen = NEW.abgebrochene_ausbildungen,
    abgebrochene_ausbildungen_text = NEW.abgebrochene_ausbildungen_text,
    abschluss_ausserbetriebliche_ausbildung = NEW.abschluss_ausserbetriebliche_ausbildung,
    abschluss_im_ausbildungsberuf_1 = NEW.abschluss_im_ausbildungsberuf_1,
    ausbildungsdauer_1 = NEW.ausbildungsdauer_1,
    wann_abgeschlossen_1 = NEW.wann_abgeschlossen_1,
    weitere_ausbildungen_1 = NEW.weitere_ausbildungen_1,
    weitere_ausbildungen_text_1 = NEW.weitere_ausbildungen_text_1,
    dauer_weitere_ausbildungen_1 = NEW.dauer_weitere_ausbildungen_1,
    abgebrochene_ausbildungen_1 = NEW.abgebrochene_ausbildungen_1,
    abgebrochene_ausbildungen_text_1 = NEW.abgebrochene_ausbildungen_text_1,
    massnahmeteilnahme = NEW.massnahmeteilnahme,
    letzte_massnahme = NEW.letzte_massnahme,
    finanzierung = NEW.finanzierung,
    austrittsdatum = NEW.austrittsdatum,
    dauer = NEW.dauer,
    weitere_massnahmen = NEW.weitere_massnahmen,
    massnahme_1 = NEW.massnahme_1,
    massnahme1_finanzierung = NEW.massnahme1_finanzierung,
    massnahme_2 = NEW.massnahme_2,
    massnahme2_finanzierung = NEW.massnahme2_finanzierung,
    massnahme_3 = NEW.massnahme_3,
    massnahme3_finanzierung = NEW.massnahme3_finanzierung,
    abgebrochene_massnahmen = NEW.abgebrochene_massnahmen,
    erwerbstaetigkeit_job = NEW.erwerbstaetigkeit_job,
    letzte_taetigkeit = NEW.letzte_taetigkeit,
    dauer_1 = NEW.dauer_1,
    austrittsdatum_1 = NEW.austrittsdatum_1,
    weitere_taetigkeiten = NEW.weitere_taetigkeiten,
    weitere_jobs_text = NEW.weitere_jobs_text,
    dauer_weitere_taetigkeiten = NEW.dauer_weitere_taetigkeiten,
    arbeitslosigkeit_1 = NEW.arbeitslosigkeit_1,
    anzahl_arbeitslosigkeit = NEW.anzahl_arbeitslosigkeit,
    dauer_nach_letzter_beschaeftigung = NEW.dauer_nach_letzter_beschaeftigung,
    dauer_insgesamt = NEW.dauer_insgesamt,
    wehrpflicht_zivildienst_freiwilligenjahr_1 = NEW.wehrpflicht_zivildienst_freiwilligenjahr_1,
    art_des_dienstes = NEW.art_des_dienstes,
    dauer_2 = NEW.dauer_2,
    mutterschutz_erziehungsurlaub = NEW.mutterschutz_erziehungsurlaub,
    anzahl = NEW.anzahl,
    dauer_letzter_erziehungsurlaub = NEW.dauer_letzter_erziehungsurlaub,
    dauer_insgesamt_1 = NEW.dauer_insgesamt_1,
    kompetenzfeststellung = NEW.kompetenzfeststellung,
    zielsetzung = NEW.zielsetzung,
    begleitung = NEW.begleitung,
    welche_vermittlung_nennung = NEW.welche_vermittlung_nennung,
    freie_dokumentation = NEW.freie_dokumentation,
    bildender_bereich = NEW.bildender_bereich,
    bereich_berufliche_qualifizierung = NEW.bereich_berufliche_qualifizierung,
    bereich_berufsvorbereitung = NEW.bereich_berufsvorbereitung,
    bereich_lebensbewaeltigung = NEW.bereich_lebensbewaeltigung,
    sonstiger_foerderbedarf = NEW.sonstiger_foerderbedarf,
    sonstiger_foerderbedarf_nennung = NEW.sonstiger_foerderbedarf_nennung,
    foerderzeitraum_start = NEW.foerderzeitraum_start,
    foerderzeitraum_ende = NEW.foerderzeitraum_ende,
    dokumentation_zum_foerderplan = NEW.dokumentation_zum_foerderplan,
    anzahl_kontakte_termine = NEW.anzahl_kontakte_termine,
    anzahl_der_stunden = NEW.anzahl_der_stunden,
    zeitraum_fallmanagement = NEW.zeitraum_fallmanagement,
    beendigung_5 = NEW.beendigung_5,
    art_beendigung = NEW.art_beendigung,
    abbruch_durch = NEW.abbruch_durch,
    gruende_abbruch = NEW.gruende_abbruch,
    gruende_sonstiges_nennung_4 = NEW.gruende_sonstiges_nennung_4,
    vermittlung = NEW.vermittlung,
    vermittlung_nennung = NEW.vermittlung_nennung,
    veraenderungen_rechtskreis = NEW.veraenderungen_rechtskreis,
    allgemein_bildender_schulabschluss = NEW.allgemein_bildender_schulabschluss,
    allgemein_bildender_schulabschluss_sonstiges_nennung = NEW.allgemein_bildender_schulabschluss_sonstiges_nennung,
    massnahmeabschluss = NEW.massnahmeabschluss,
    schulart = NEW.schulart,
    schule_sonstiges_nennung = NEW.schule_sonstiges_nennung,
    klassenstufe_1 = NEW.klassenstufe_1,
    ausbildung = NEW.ausbildung,
    ausbildung_sonstiges_nennung = NEW.ausbildung_sonstiges_nennung,
    umschulung = NEW.umschulung,
    umschulung_nennung = NEW.umschulung_nennung,
    arbeitstaetigkeit = NEW.arbeitstaetigkeit,
    erwerbstatus = NEW.erwerbstatus,
    erwerbsstatus_sonstiges_nennung = NEW.erwerbsstatus_sonstiges_nennung,
    beschaeftigungsumfang = NEW.beschaeftigungsumfang,
    kategorien_nichterwerbsarbeit_nichtausbildung = NEW.kategorien_nichterwerbsarbeit_nichtausbildung,
    kategorien_sonstiges_nennung = NEW.kategorien_sonstiges_nennung,
    zeitraum_nachbetreuung = NEW.zeitraum_nachbetreuung,
    verbleib_nachbetreuung = NEW.verbleib_nachbetreuung,
    mitgliedschaft_organisationen_vereinen = NEW.mitgliedschaft_organisationen_vereinen,
    intensitaet_mitgliedschaft = NEW.intensitaet_mitgliedschaft,
    nutzung_kulturangeboten = NEW.nutzung_kulturangeboten,
    haeufigkeit = NEW.haeufigkeit,
    soziales_engagement = NEW.soziales_engagement,
    unternehmungen_freunden = NEW.unternehmungen_freunden,
    unternehmungen_familie = NEW.unternehmungen_familie,
    peergroup = NEW.peergroup,
    familie_1 = NEW.familie_1,
    beziehung = NEW.beziehung,
    arbeit_schule = NEW.arbeit_schule,
    bester_freund_freundin = NEW.bester_freund_freundin,
    einschaetzung_zielerreichung = NEW.einschaetzung_zielerreichung,
    erreichte_qualifikation = NEW.erreichte_qualifikation,
    integration_ist_nachhaltig = NEW.integration_ist_nachhaltig,
    selbststaendige_berufswegplanung = NEW.selbststaendige_berufswegplanung,
    eigenstaendige_lebensgestaltung = NEW.eigenstaendige_lebensgestaltung,
    soziale_bezugssysteme = NEW.soziale_bezugssysteme,
    teilnahme_gesellschaftliches_leben = NEW.teilnahme_gesellschaftliches_leben,
    freie_dokumentation_abschlussbewertung = NEW.freie_dokumentation_abschlussbewertung
WHERE id = NEW.id;

CREATE OR REPLACE RULE master_tbl_delete_rule AS ON DELETE TO master_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE master_tbl_insert_rule AS ON INSERT TO master_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON master_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON master_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON master_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW master_tbl_anonym_view
AS SELECT
    id,
    uuid_id,
    bearbeiter_id,
    einverstaendniserklaerung,
    NULL::VARCHAR(60) AS fkz,
    NULL::VARCHAR(60) AS name_kompetenzagentur,
    NULL::VARCHAR(60) AS vorname,
    hash_value(name, vorname) AS name,
    erstgespraech,
    zugang_kompetenzagentur,
    NULL::VARCHAR(100) AS kundennummer,
    schulpflichtig,
    noch_schueler,
    bei_eintritt_ist_der_jugendliche,
    rechtlicher_kontext,
    schnittstelle_zu,
    NULL::VARCHAR(100) AS kontaktdaten_arge,
    NULL::VARCHAR(100) AS kundennummer_arge,
    NULL::VARCHAR(100) AS kontaktdaten_agentur_fuer_arbeit,
    NULL::VARCHAR(100) AS kundennummer_agentur_fuer_arbeit,
    NULL::VARCHAR(100) AS kundennummer_jugendamt,
    NULL::VARCHAR(100) AS kontaktdaten_jugendamt,
    NULL::VARCHAR(100) AS kontaktdaten_schule,
    NULL::VARCHAR(100) AS schulname,
    NULL::VARCHAR(100) AS kontaktdaten_gerichtshilfe,
    NULL::VARCHAR(30) AS kundennummer_gerichtshilfe,
    NULL::TEXT AS kontaktdaten_weitere_institutionen,
    NULL::VARCHAR(60) AS vorname_1,
    NULL::VARCHAR(60) AS name_1,
    geschlecht,
    get_age(geburtsdatum) AS geburtsdatum,
    familienstand,
    NULL::VARCHAR(200) AS strasse,
    shorten_value(plz, 3) AS plz,
    NULL::VARCHAR(60) AS wohnort,
    groesse_wohnort,
    NULL::VARCHAR(60) AS vorwahl,
    NULL::VARCHAR(60) AS telefon_1,
    NULL::VARCHAR(120) AS mobiltelefon,
    NULL::VARCHAR(120) AS email,
    NULL::TEXT AS telefon_sonstige,
    NULL::VARCHAR(120) AS zweiter_wohnort,
    geburtsland,
    NULL::VARCHAR(60) AS geburtsland_sonstige_nennung,
    muttersprache,
    NULL::VARCHAR(60) AS muttersprache_sonstige_nennung,
    staatsangehoerigkeit,
    NULL::VARCHAR(60) AS staatsangehoerigkeit_sonstige_nennung,
    aufenthaltsstatus,
    aufenthaltsdauer,
    vater,
    mutter,
    familiaere_situation_1,
    NULL::VARCHAR(60) AS vorname_2,
    NULL::VARCHAR(60) AS name_2,
    NULL::VARCHAR(60) AS vorwahl_1,
    NULL::VARCHAR(60) AS telefon_1_1,
    NULL::VARCHAR(60) AS mobiltelefon_1,
    NULL::VARCHAR(120) AS email_1,
    geschwister,
    anzahl_kinder,
    haushalt_jugendlicher,
    alleinerziehend,
    familie_selbststaendig,
    familie_staatliche_unterstuetzung,
    jugendlicher_selbststaendig,
    jugendlicher_staatliche_unterstuetzung,
    jugendlicher_unterstuetzung_soziales_umfeld,
    wohnaufenthalt,
    arzt_gutachten_physisch,
    check_bool(physische_beeintraechtigungen, arzt_gutachten_physisch) AS physische_beeintraechtigungen,
    check_bool(physische_beeintraechtigungen_text, arzt_gutachten_physisch) AS physische_beeintraechtigungen_text,
    arzt_gutachten_psychisch,
    check_bool(psychische_beeintraechtigungen, arzt_gutachten_psychisch) AS psychische_beeintraechtigungen,
    check_bool(psychische_beeintraechtigungen_text, arzt_gutachten_psychisch) AS psychische_beeintraechtigungen_text,
    mutter_2,
    vater_2,
    grosseltern,
    geschwister_2,
    pflegefamilie_1,
    stiefmutter,
    stiefvater,
    verwandte_1,
    partner,
    heimbetreuerin,
    betreuerin,
    lehrerin,
    mitbewohner,
    partner_mutter,
    partner_vater,
    freund,
    keine_bezugsperson,
    hobbies_1,
    hobbies_text,
    mitarbeit_organisationen,
    organisationen_vereine_text,
    sonstiges_1,
    sonstiges_text,
    clique,
    freundin,
    parterin,
    schule_2,
    klassenstufe,
    klassenwiederholung,
    abgang,
    nicht_anwesende_tage,
    unentschuldigte_tage,
    schulabschluss_1,
    schulabschlusserwerbsland,
    NULL::VARCHAR(60) AS schulabschlusserwerbsland_sonstige_nennung,
    abschluss_betriebliche_ausbildung,
    abschluss_im_ausbildungsberuf,
    ausbildungsdauer,
    wann_abgeschlossen,
    weitere_ausbildungen,
    weitere_ausbildungen_text,
    dauer_weitere_ausbildungen,
    abgebrochene_ausbildungen,
    NULL::TEXT AS abgebrochene_ausbildungen_text,
    abschluss_ausserbetriebliche_ausbildung,
    abschluss_im_ausbildungsberuf_1,
    ausbildungsdauer_1,
    wann_abgeschlossen_1,
    weitere_ausbildungen_1,
    NULL::TEXT AS weitere_ausbildungen_text_1,
    dauer_weitere_ausbildungen_1,
    abgebrochene_ausbildungen_1,
    NULL::TEXT AS abgebrochene_ausbildungen_text_1,
    massnahmeteilnahme,
    NULL::VARCHAR(60) AS letzte_massnahme,
    finanzierung,
    austrittsdatum,
    dauer,
    weitere_massnahmen,
    massnahme_1,
    massnahme1_finanzierung,
    massnahme_2,
    massnahme2_finanzierung,
    massnahme_3,
    massnahme3_finanzierung,
    abgebrochene_massnahmen,
    erwerbstaetigkeit_job,
    NULL::VARCHAR(60) AS letzte_taetigkeit,
    dauer_1,
    austrittsdatum_1,
    weitere_taetigkeiten,
    weitere_jobs_text,
    dauer_weitere_taetigkeiten,
    arbeitslosigkeit_1,
    anzahl_arbeitslosigkeit,
    dauer_nach_letzter_beschaeftigung,
    dauer_insgesamt,
    wehrpflicht_zivildienst_freiwilligenjahr_1,
    art_des_dienstes,
    dauer_2,
    mutterschutz_erziehungsurlaub,
    anzahl,
    dauer_letzter_erziehungsurlaub,
    dauer_insgesamt_1,
    kompetenzfeststellung,
    zielsetzung,
    begleitung,
    welche_vermittlung_nennung,
    NULL::TEXT AS freie_dokumentation,
    bildender_bereich,
    bereich_berufliche_qualifizierung,
    bereich_berufsvorbereitung,
    bereich_lebensbewaeltigung,
    sonstiger_foerderbedarf,
    NULL::VARCHAR(120) AS sonstiger_foerderbedarf_nennung,
    foerderzeitraum_start,
    foerderzeitraum_ende,
    NULL::TEXT AS dokumentation_zum_foerderplan,
    anzahl_kontakte_termine,
    anzahl_der_stunden,
    zeitraum_fallmanagement,
    beendigung_5,
    art_beendigung,
    abbruch_durch,
    gruende_abbruch,
    NULL::VARCHAR(60) AS gruende_sonstiges_nennung_4,
    vermittlung,
    vermittlung_nennung,
    veraenderungen_rechtskreis,
    allgemein_bildender_schulabschluss,
    allgemein_bildender_schulabschluss_sonstiges_nennung,
    massnahmeabschluss,
    schulart,
    schule_sonstiges_nennung,
    NULL::INTEGER AS klassenstufe_1,
    ausbildung,
    ausbildung_sonstiges_nennung,
    umschulung,
    umschulung_nennung,
    arbeitstaetigkeit,
    erwerbstatus,
    erwerbsstatus_sonstiges_nennung,
    beschaeftigungsumfang,
    kategorien_nichterwerbsarbeit_nichtausbildung,
    kategorien_sonstiges_nennung,
    zeitraum_nachbetreuung,
    verbleib_nachbetreuung,
    mitgliedschaft_organisationen_vereinen,
    intensitaet_mitgliedschaft,
    nutzung_kulturangeboten,
    haeufigkeit,
    soziales_engagement,
    unternehmungen_freunden,
    unternehmungen_familie,
    peergroup,
    familie_1,
    beziehung,
    arbeit_schule,
    bester_freund_freundin,
    einschaetzung_zielerreichung,
    erreichte_qualifikation,
    integration_ist_nachhaltig,
    selbststaendige_berufswegplanung,
    eigenstaendige_lebensgestaltung,
    soziale_bezugssysteme,
    teilnahme_gesellschaftliches_leben,
    NULL::TEXT AS freie_dokumentation_abschlussbewertung FROM master_tbl_view;

CREATE OR REPLACE RULE master_tbl_anonym_delete_rule AS ON DELETE TO master_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE master_tbl_anonym_insert_rule AS ON INSERT TO master_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON master_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON master_tbl_anonym_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_kompetenzfestellung_tbl_view
AS SELECT rg_kompetenzfestellung_tbl.* FROM rg_kompetenzfestellung_tbl
    INNER JOIN master_tbl
    ON         rg_kompetenzfestellung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE rg_kompetenzfestellung_tbl_update_rule AS ON UPDATE TO rg_kompetenzfestellung_tbl_view
DO INSTEAD UPDATE rg_kompetenzfestellung_tbl SET
    verfahren_kompetenzfeststellung = NEW.verfahren_kompetenzfeststellung,
    externeranbieter = NEW.externeranbieter,
    assessment_center = NEW.assessment_center,
    interview = NEW.interview,
    biografieorientierte_methoden = NEW.biografieorientierte_methoden,
    selbsttest_selbstbeurteilung = NEW.selbsttest_selbstbeurteilung,
    sonstiges_2 = NEW.sonstiges_2,
    artdesverfahrens_nennung_sonstiges = NEW.artdesverfahrens_nennung_sonstiges,
    testverfahren = NEW.testverfahren,
    intelligenztest = NEW.intelligenztest,
    intelligenztest_nennung = NEW.intelligenztest_nennung,
    schultest = NEW.schultest,
    schultest_nennung = NEW.schultest_nennung,
    leistungstest = NEW.leistungstest,
    leistungstest_nennung = NEW.leistungstest_nennung,
    persoenlichkeitstest = NEW.persoenlichkeitstest,
    persoenlichkeitstest_nennung = NEW.persoenlichkeitstest_nennung,
    einstellungs_interessentest = NEW.einstellungs_interessentest,
    einstellungs_interessentest_nennung = NEW.einstellungs_interessentest_nennung,
    dauer_des_verfahrens = NEW.dauer_des_verfahrens,
    rechenfaehigkeit = NEW.rechenfaehigkeit,
    raeumliches_vorstellungsvermoegen = NEW.raeumliches_vorstellungsvermoegen,
    grundverstaendnis_naturwissenschaftlicher_zusammenhaenge = NEW.grundverstaendnis_naturwissenschaftlicher_zusammenhaenge,
    logisches_denken = NEW.logisches_denken,
    wirtschaftliche_grundkenntnisse = NEW.wirtschaftliche_grundkenntnisse,
    allgemeine_pc_kenntnisse = NEW.allgemeine_pc_kenntnisse,
    spezielle_pc_kenntnisse_grafikbearbeitung_musikbearbeitung = NEW.spezielle_pc_kenntnisse_grafikbearbeitung_musikbearbeitung,
    internet_anwendungen_und_e_mail_kenntnisse = NEW.internet_anwendungen_und_e_mail_kenntnisse,
    andere_medien = NEW.andere_medien,
    hand_fingergeschick = NEW.hand_fingergeschick,
    formauffassung = NEW.formauffassung,
    umgang_maschinen_werkzeugen = NEW.umgang_maschinen_werkzeugen,
    gestalterische_fantasie = NEW.gestalterische_fantasie,
    methoden_der_informationsgewinnung = NEW.methoden_der_informationsgewinnung,
    methoden_der_informationsverarbeitung = NEW.methoden_der_informationsverarbeitung,
    einsatz_lern_und_arbeitstechniken = NEW.einsatz_lern_und_arbeitstechniken,
    problemloesungsorientierung = NEW.problemloesungsorientierung,
    texte_lesen_und_verstehen = NEW.texte_lesen_und_verstehen,
    texte_schreiben_und_formulieren = NEW.texte_schreiben_und_formulieren,
    deutsche_sprache = NEW.deutsche_sprache,
    sprache1 = NEW.sprache1,
    sprache1_sonstige_nennung = NEW.sprache1_sonstige_nennung,
    beherrschung_sprache1 = NEW.beherrschung_sprache1,
    sprache2 = NEW.sprache2,
    sprache2_sonstige_nennung = NEW.sprache2_sonstige_nennung,
    beherrschung_sprache2 = NEW.beherrschung_sprache2,
    sprache3 = NEW.sprache3,
    sprache3_sonstige_nennung = NEW.sprache3_sonstige_nennung,
    beherrschung_sprache3 = NEW.beherrschung_sprache3,
    kommunikationsfaehigkeit = NEW.kommunikationsfaehigkeit,
    kritikfaehigkeit = NEW.kritikfaehigkeit,
    konfliktfaehigkeit = NEW.konfliktfaehigkeit,
    kreativitaet = NEW.kreativitaet,
    team_und_kooperationsfaehigkeit = NEW.team_und_kooperationsfaehigkeit,
    solidarisches_verhalten = NEW.solidarisches_verhalten,
    selbstaendiges_lernen_arbeiten = NEW.selbstaendiges_lernen_arbeiten,
    selbstreflexion_selbsteinschaetzung = NEW.selbstreflexion_selbsteinschaetzung,
    verantwortungsuebernahme = NEW.verantwortungsuebernahme,
    zielorientierung = NEW.zielorientierung,
    zuverlaessigkeit = NEW.zuverlaessigkeit,
    altersgerechter_entwicklungsstand = NEW.altersgerechter_entwicklungsstand,
    gesundheitliche_voraussetzungen = NEW.gesundheitliche_voraussetzungen,
    konzentration = NEW.konzentration,
    ausdauer = NEW.ausdauer,
    durchhaltevermoegen = NEW.durchhaltevermoegen,
    merkfaehigkeit = NEW.merkfaehigkeit,
    bearbeitungsgeschwindigkeit = NEW.bearbeitungsgeschwindigkeit,
    befaehigung_zu_daueraufmerksamkeit = NEW.befaehigung_zu_daueraufmerksamkeit,
    leistungsbereitschaft = NEW.leistungsbereitschaft,
    sorgfalt = NEW.sorgfalt,
    umgangsformen = NEW.umgangsformen,
    dokumentation_kompetenzfeststellung = NEW.dokumentation_kompetenzfeststellung
WHERE id = NEW.id;

CREATE OR REPLACE RULE rg_kompetenzfestellung_tbl_delete_rule AS ON DELETE TO rg_kompetenzfestellung_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_kompetenzfestellung_tbl_insert_rule AS ON INSERT TO rg_kompetenzfestellung_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_kompetenzfestellung_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON rg_kompetenzfestellung_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_kompetenzfestellung_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_kompetenzfestellung_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    verfahren_kompetenzfeststellung,
    externeranbieter,
    assessment_center,
    interview,
    biografieorientierte_methoden,
    selbsttest_selbstbeurteilung,
    sonstiges_2,
    artdesverfahrens_nennung_sonstiges,
    testverfahren,
    intelligenztest,
    intelligenztest_nennung,
    schultest,
    schultest_nennung,
    leistungstest,
    leistungstest_nennung,
    persoenlichkeitstest,
    persoenlichkeitstest_nennung,
    einstellungs_interessentest,
    einstellungs_interessentest_nennung,
    dauer_des_verfahrens,
    rechenfaehigkeit,
    raeumliches_vorstellungsvermoegen,
    grundverstaendnis_naturwissenschaftlicher_zusammenhaenge,
    logisches_denken,
    wirtschaftliche_grundkenntnisse,
    allgemeine_pc_kenntnisse,
    spezielle_pc_kenntnisse_grafikbearbeitung_musikbearbeitung,
    internet_anwendungen_und_e_mail_kenntnisse,
    andere_medien,
    hand_fingergeschick,
    formauffassung,
    umgang_maschinen_werkzeugen,
    gestalterische_fantasie,
    methoden_der_informationsgewinnung,
    methoden_der_informationsverarbeitung,
    einsatz_lern_und_arbeitstechniken,
    problemloesungsorientierung,
    texte_lesen_und_verstehen,
    texte_schreiben_und_formulieren,
    deutsche_sprache,
    sprache1,
    sprache1_sonstige_nennung,
    beherrschung_sprache1,
    sprache2,
    sprache2_sonstige_nennung,
    beherrschung_sprache2,
    sprache3,
    sprache3_sonstige_nennung,
    beherrschung_sprache3,
    kommunikationsfaehigkeit,
    kritikfaehigkeit,
    konfliktfaehigkeit,
    kreativitaet,
    team_und_kooperationsfaehigkeit,
    solidarisches_verhalten,
    selbstaendiges_lernen_arbeiten,
    selbstreflexion_selbsteinschaetzung,
    verantwortungsuebernahme,
    zielorientierung,
    zuverlaessigkeit,
    altersgerechter_entwicklungsstand,
    gesundheitliche_voraussetzungen,
    konzentration,
    ausdauer,
    durchhaltevermoegen,
    merkfaehigkeit,
    bearbeitungsgeschwindigkeit,
    befaehigung_zu_daueraufmerksamkeit,
    leistungsbereitschaft,
    sorgfalt,
    umgangsformen,
    NULL::TEXT AS dokumentation_kompetenzfeststellung FROM rg_kompetenzfestellung_tbl_view;

CREATE OR REPLACE RULE rg_kompetenzfestellung_tbl_anonym_delete_rule AS ON DELETE TO rg_kompetenzfestellung_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_kompetenzfestellung_tbl_anonym_insert_rule AS ON INSERT TO rg_kompetenzfestellung_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_kompetenzfestellung_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_kompetenzfestellung_tbl_anonym_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_angebote_bildenden_bereich_tbl_view
AS SELECT rg_angebote_bildenden_bereich_tbl.* FROM rg_angebote_bildenden_bereich_tbl
    INNER JOIN master_tbl
    ON         rg_angebote_bildenden_bereich_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE rg_angebote_bildenden_bereich_tbl_update_rule AS ON UPDATE TO rg_angebote_bildenden_bereich_tbl_view
DO INSTEAD UPDATE rg_angebote_bildenden_bereich_tbl SET
    angebotsform = NEW.angebotsform,
    art_der_unterstuetzung = NEW.art_der_unterstuetzung,
    durchfuehrende_institution = NEW.durchfuehrende_institution,
    unterstuetzung_start = NEW.unterstuetzung_start,
    unterstuetzung_ende = NEW.unterstuetzung_ende,
    zeitlicher_umfang_unterstuetzung = NEW.zeitlicher_umfang_unterstuetzung,
    beendigung_1 = NEW.beendigung_1,
    beendigung_durch = NEW.beendigung_durch,
    gruende_vorzeitige_beendigung = NEW.gruende_vorzeitige_beendigung,
    gruende_sonstiges_nennung = NEW.gruende_sonstiges_nennung,
    nachweis_bescheinigung = NEW.nachweis_bescheinigung,
    sonstiges_nachweis_bescheinigung = NEW.sonstiges_nachweis_bescheinigung
WHERE id = NEW.id;

CREATE OR REPLACE RULE rg_angebote_bildenden_bereich_tbl_delete_rule AS ON DELETE TO rg_angebote_bildenden_bereich_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_angebote_bildenden_bereich_tbl_insert_rule AS ON INSERT TO rg_angebote_bildenden_bereich_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_angebote_bildenden_bereich_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON rg_angebote_bildenden_bereich_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_angebote_bildenden_bereich_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_angebote_bildenden_bereich_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    angebotsform,
    art_der_unterstuetzung,
    durchfuehrende_institution,
    unterstuetzung_start,
    unterstuetzung_ende,
    zeitlicher_umfang_unterstuetzung,
    beendigung_1,
    beendigung_durch,
    gruende_vorzeitige_beendigung,
    NULL::VARCHAR(120) AS gruende_sonstiges_nennung,
    nachweis_bescheinigung,
    NULL::VARCHAR(120) AS sonstiges_nachweis_bescheinigung FROM rg_angebote_bildenden_bereich_tbl_view;

CREATE OR REPLACE RULE rg_angebote_bildenden_bereich_tbl_anonym_delete_rule AS ON DELETE TO rg_angebote_bildenden_bereich_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_angebote_bildenden_bereich_tbl_anonym_insert_rule AS ON INSERT TO rg_angebote_bildenden_bereich_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_angebote_bildenden_bereich_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_angebote_bildenden_bereich_tbl_anonym_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_angebote_berufsvorbereitung_tbl_view
AS SELECT rg_angebote_berufsvorbereitung_tbl.* FROM rg_angebote_berufsvorbereitung_tbl
    INNER JOIN master_tbl
    ON         rg_angebote_berufsvorbereitung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE rg_angebote_berufsvorbereitung_tbl_update_rule AS ON UPDATE TO rg_angebote_berufsvorbereitung_tbl_view
DO INSTEAD UPDATE rg_angebote_berufsvorbereitung_tbl SET
    angebotsform_1 = NEW.angebotsform_1,
    art_der_unterstuetzung_1 = NEW.art_der_unterstuetzung_1,
    unterstuetzung_sonstiges_nennung = NEW.unterstuetzung_sonstiges_nennung,
    durchfuehrende_institution_1 = NEW.durchfuehrende_institution_1,
    unterstuetzung_start_1 = NEW.unterstuetzung_start_1,
    unterstuetzung_ende_1 = NEW.unterstuetzung_ende_1,
    zeitlicher_umfang_unterstuetzung_1 = NEW.zeitlicher_umfang_unterstuetzung_1,
    beendigung_2 = NEW.beendigung_2,
    beendigung_durch_1 = NEW.beendigung_durch_1,
    gruende_vorzeitige_beendigung_1 = NEW.gruende_vorzeitige_beendigung_1,
    gruende_sonstiges_nennung_1 = NEW.gruende_sonstiges_nennung_1,
    nachweis_bescheinigung_1 = NEW.nachweis_bescheinigung_1,
    sonstiges_nachweis_bescheinigung_1 = NEW.sonstiges_nachweis_bescheinigung_1
WHERE id = NEW.id;

CREATE OR REPLACE RULE rg_angebote_berufsvorbereitung_tbl_delete_rule AS ON DELETE TO rg_angebote_berufsvorbereitung_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_angebote_berufsvorbereitung_tbl_insert_rule AS ON INSERT TO rg_angebote_berufsvorbereitung_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_angebote_berufsvorbereitung_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON rg_angebote_berufsvorbereitung_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_angebote_berufsvorbereitung_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_angebote_berufsvorbereitung_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    angebotsform_1,
    art_der_unterstuetzung_1,
    unterstuetzung_sonstiges_nennung,
    durchfuehrende_institution_1,
    unterstuetzung_start_1,
    unterstuetzung_ende_1,
    zeitlicher_umfang_unterstuetzung_1,
    beendigung_2,
    beendigung_durch_1,
    gruende_vorzeitige_beendigung_1,
    NULL::VARCHAR(120) AS gruende_sonstiges_nennung_1,
    nachweis_bescheinigung_1,
    NULL::VARCHAR(120) AS sonstiges_nachweis_bescheinigung_1 FROM rg_angebote_berufsvorbereitung_tbl_view;

CREATE OR REPLACE RULE rg_angebote_berufsvorbereitung_tbl_anonym_delete_rule AS ON DELETE TO rg_angebote_berufsvorbereitung_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_angebote_berufsvorbereitung_tbl_anonym_insert_rule AS ON INSERT TO rg_angebote_berufsvorbereitung_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_angebote_berufsvorbereitung_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_angebote_berufsvorbereitung_tbl_anonym_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_angebote_berufliche_qualifizierung_tbl_view
AS SELECT rg_angebote_berufliche_qualifizierung_tbl.* FROM rg_angebote_berufliche_qualifizierung_tbl
    INNER JOIN master_tbl
    ON         rg_angebote_berufliche_qualifizierung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE rg_angebote_berufliche_qualifizierung_tbl_update_rule AS ON UPDATE TO rg_angebote_berufliche_qualifizierung_tbl_view
DO INSTEAD UPDATE rg_angebote_berufliche_qualifizierung_tbl SET
    angebotsform_2 = NEW.angebotsform_2,
    angebote_im_bereich = NEW.angebote_im_bereich,
    unterstuetzung_sonstiges_nennung_1 = NEW.unterstuetzung_sonstiges_nennung_1,
    durchfuehrende_institution_2 = NEW.durchfuehrende_institution_2,
    unterstuetzung_start_2 = NEW.unterstuetzung_start_2,
    unterstuetzung_ende_2 = NEW.unterstuetzung_ende_2,
    zeitlicher_umfang_unterstuetzung_2 = NEW.zeitlicher_umfang_unterstuetzung_2,
    beendigung_3 = NEW.beendigung_3,
    beendigung_durch_2 = NEW.beendigung_durch_2,
    gruende_vorzeitige_beendigung_2 = NEW.gruende_vorzeitige_beendigung_2,
    gruende_sonstiges_nennung_2 = NEW.gruende_sonstiges_nennung_2,
    nachweis_bescheinigung_2 = NEW.nachweis_bescheinigung_2,
    sonstiges_nachweis_bescheinigung_2 = NEW.sonstiges_nachweis_bescheinigung_2
WHERE id = NEW.id;

CREATE OR REPLACE RULE rg_angebote_berufliche_qualifizierung_tbl_delete_rule AS ON DELETE TO rg_angebote_berufliche_qualifizierung_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_angebote_berufliche_qualifizierung_tbl_insert_rule AS ON INSERT TO rg_angebote_berufliche_qualifizierung_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_angebote_berufliche_qualifizierung_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON rg_angebote_berufliche_qualifizierung_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_angebote_berufliche_qualifizierung_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_angebote_berufliche_qualifizierung_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    angebotsform_2,
    angebote_im_bereich,
    unterstuetzung_sonstiges_nennung_1,
    durchfuehrende_institution_2,
    unterstuetzung_start_2,
    unterstuetzung_ende_2,
    zeitlicher_umfang_unterstuetzung_2,
    beendigung_3,
    beendigung_durch_2,
    gruende_vorzeitige_beendigung_2,
    NULL::VARCHAR(120) AS gruende_sonstiges_nennung_2,
    nachweis_bescheinigung_2,
    NULL::VARCHAR(120) AS sonstiges_nachweis_bescheinigung_2 FROM rg_angebote_berufliche_qualifizierung_tbl_view;

CREATE OR REPLACE RULE rg_angebote_berufliche_qualifizierung_tbl_anonym_delete_rule AS ON DELETE TO rg_angebote_berufliche_qualifizierung_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_angebote_berufliche_qualifizierung_tbl_anonym_insert_rule AS ON INSERT TO rg_angebote_berufliche_qualifizierung_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_angebote_berufliche_qualifizierung_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_angebote_berufliche_qualifizierung_tbl_anonym_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_angebote_lebensbewaeltigung_tbl_view
AS SELECT rg_angebote_lebensbewaeltigung_tbl.* FROM rg_angebote_lebensbewaeltigung_tbl
    INNER JOIN master_tbl
    ON         rg_angebote_lebensbewaeltigung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin_ka','MEMBER');

CREATE OR REPLACE RULE rg_angebote_lebensbewaeltigung_tbl_update_rule AS ON UPDATE TO rg_angebote_lebensbewaeltigung_tbl_view
DO INSTEAD UPDATE rg_angebote_lebensbewaeltigung_tbl SET
    angebotsform_3 = NEW.angebotsform_3,
    unterstuetzung_angebotsform_sonstiges_nennung = NEW.unterstuetzung_angebotsform_sonstiges_nennung,
    art_der_unterstuetzung_2 = NEW.art_der_unterstuetzung_2,
    unterstuetzung_sonstiges_nennung_2 = NEW.unterstuetzung_sonstiges_nennung_2,
    durchfuehrende_institution_3 = NEW.durchfuehrende_institution_3,
    unterstuetzung_start_3 = NEW.unterstuetzung_start_3,
    unterstuetzung_ende_3 = NEW.unterstuetzung_ende_3,
    zeitlicher_umfang_unterstuetzung_3 = NEW.zeitlicher_umfang_unterstuetzung_3,
    beendigung_4 = NEW.beendigung_4,
    beendigung_durch_3 = NEW.beendigung_durch_3,
    gruende_vorzeitige_beendigung_3 = NEW.gruende_vorzeitige_beendigung_3,
    gruende_sonstiges_nennung_3 = NEW.gruende_sonstiges_nennung_3,
    nachweis_bescheinigung_3 = NEW.nachweis_bescheinigung_3,
    sonstiges_nachweis_bescheinigung_3 = NEW.sonstiges_nachweis_bescheinigung_3
WHERE id = NEW.id;

CREATE OR REPLACE RULE rg_angebote_lebensbewaeltigung_tbl_delete_rule AS ON DELETE TO rg_angebote_lebensbewaeltigung_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_angebote_lebensbewaeltigung_tbl_insert_rule AS ON INSERT TO rg_angebote_lebensbewaeltigung_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_angebote_lebensbewaeltigung_tbl_view TO GROUP :cm_ka_group;
GRANT UPDATE ON rg_angebote_lebensbewaeltigung_tbl_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_angebote_lebensbewaeltigung_tbl_view TO GROUP :admin_ka_group;

CREATE OR REPLACE VIEW rg_angebote_lebensbewaeltigung_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    angebotsform_3,
    unterstuetzung_angebotsform_sonstiges_nennung,
    art_der_unterstuetzung_2,
    unterstuetzung_sonstiges_nennung_2,
    durchfuehrende_institution_3,
    unterstuetzung_start_3,
    unterstuetzung_ende_3,
    zeitlicher_umfang_unterstuetzung_3,
    beendigung_4,
    beendigung_durch_3,
    gruende_vorzeitige_beendigung_3,
    NULL::VARCHAR(120) AS gruende_sonstiges_nennung_3,
    nachweis_bescheinigung_3,
    NULL::VARCHAR(120) AS sonstiges_nachweis_bescheinigung_3 FROM rg_angebote_lebensbewaeltigung_tbl_view;

CREATE OR REPLACE RULE rg_angebote_lebensbewaeltigung_tbl_anonym_delete_rule AS ON DELETE TO rg_angebote_lebensbewaeltigung_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_angebote_lebensbewaeltigung_tbl_anonym_insert_rule AS ON INSERT TO rg_angebote_lebensbewaeltigung_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_angebote_lebensbewaeltigung_tbl_anonym_view TO GROUP :cm_ka_group;
GRANT SELECT ON rg_angebote_lebensbewaeltigung_tbl_anonym_view TO GROUP :admin_ka_group;

--
-- Function to anonymize a case and all of its repeat groups
--
CREATE OR REPLACE FUNCTION anonymize_case(mid int)
RETURNS void AS
$$
BEGIN
    -- anonymize master table
    UPDATE master_tbl_view SET
        fkz = NULL,
        name_kompetenzagentur = NULL,
        vorname = NULL,
        name = hash_value(name, vorname),
        kundennummer = NULL,
        kontaktdaten_arge = NULL,
        kundennummer_arge = NULL,
        kontaktdaten_agentur_fuer_arbeit = NULL,
        kundennummer_agentur_fuer_arbeit = NULL,
        kundennummer_jugendamt = NULL,
        kontaktdaten_jugendamt = NULL,
        kontaktdaten_schule = NULL,
        schulname = NULL,
        kontaktdaten_gerichtshilfe = NULL,
        kundennummer_gerichtshilfe = NULL,
        kontaktdaten_weitere_institutionen = NULL,
        vorname_1 = NULL,
        name_1 = NULL,
        geburtsdatum = get_age(geburtsdatum),
        strasse = NULL,
        plz = shorten_value(plz, 3),
        wohnort = NULL,
        vorwahl = NULL,
        telefon_1 = NULL,
        mobiltelefon = NULL,
        email = NULL,
        telefon_sonstige = NULL,
        zweiter_wohnort = NULL,
        geburtsland_sonstige_nennung = NULL,
        muttersprache_sonstige_nennung = NULL,
        staatsangehoerigkeit_sonstige_nennung = NULL,
        vorname_2 = NULL,
        name_2 = NULL,
        vorwahl_1 = NULL,
        telefon_1_1 = NULL,
        mobiltelefon_1 = NULL,
        email_1 = NULL,
        physische_beeintraechtigungen = check_bool(physische_beeintraechtigungen, arzt_gutachten_physisch),
        physische_beeintraechtigungen_text = check_bool(physische_beeintraechtigungen_text, arzt_gutachten_physisch),
        psychische_beeintraechtigungen = check_bool(psychische_beeintraechtigungen, arzt_gutachten_psychisch),
        psychische_beeintraechtigungen_text = check_bool(psychische_beeintraechtigungen_text, arzt_gutachten_psychisch),
        schulabschlusserwerbsland_sonstige_nennung = NULL,
        abgebrochene_ausbildungen_text = NULL,
        weitere_ausbildungen_text_1 = NULL,
        abgebrochene_ausbildungen_text_1 = NULL,
        letzte_massnahme = NULL,
        letzte_taetigkeit = NULL,
        freie_dokumentation = NULL,
        sonstiger_foerderbedarf_nennung = NULL,
        dokumentation_zum_foerderplan = NULL,
        gruende_sonstiges_nennung_4 = NULL,
        klassenstufe_1 = NULL,
        freie_dokumentation_abschlussbewertung = NULL
    WHERE id = mid;

    -- anonymize repeat groups
    UPDATE rg_kompetenzfestellung_tbl_view SET
        dokumentation_kompetenzfeststellung = NULL
    WHERE master_id = mid;

    UPDATE rg_angebote_bildenden_bereich_tbl_view SET
        gruende_sonstiges_nennung = NULL,
        sonstiges_nachweis_bescheinigung = NULL
    WHERE master_id = mid;

    UPDATE rg_angebote_berufsvorbereitung_tbl_view SET
        gruende_sonstiges_nennung_1 = NULL,
        sonstiges_nachweis_bescheinigung_1 = NULL
    WHERE master_id = mid;

    UPDATE rg_angebote_berufliche_qualifizierung_tbl_view SET
        gruende_sonstiges_nennung_2 = NULL,
        sonstiges_nachweis_bescheinigung_2 = NULL
    WHERE master_id = mid;

    UPDATE rg_angebote_lebensbewaeltigung_tbl_view SET
        gruende_sonstiges_nennung_3 = NULL,
        sonstiges_nachweis_bescheinigung_3 = NULL
    WHERE master_id = mid;

END;
$$ LANGUAGE plpgsql;

REVOKE ALL ON FUNCTION anonymize_case(int) FROM public;
GRANT EXECUTE ON FUNCTION anonymize_case(int) TO GROUP :cm_ka_group;

COMMIT;
