# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
# Sascha L. Teichmann <teichmann@intevation.de>
#

import traceback
import logging
import paste
import formencode

from decorator import decorator
from datetime  import datetime, date

from pylons import config

from paste.httpexceptions        import HTTPNotFound
from waskaweb.lib.base           import *
from waskaweb.lib.search         import INCONSISTENCY_CHECK_AFTER
from waskaweb.lib.evaluation     import *
from waskaweb.lib.validators     import NewCaseForm, \
                                        SetStandinForm, \
                                        SetEditorForm, \
                                        CreateAppointmentForm, \
                                        CreateLogbookForm, \
                                        EditLogbookForm, \
                                        SetPhaseForm

from waskaweb.lib.helpers import dd_mm_YYYY, HH_MM

from waskaweb.model.repeatgroup    import AidObject, AidList 
from waskaweb.model.user           import *
from waskaweb.model.statement      import *
from waskaweb.model.case           import *
from waskaweb.model.appointment    import CaseAppointmentOverview, \
                                          CaseAppointmentFactory, \
                                          CaseReminderOverview

from waskaweb.controllers.CaseBase import CasebaseController
from waskaweb.model.semantic       import *
from waskaweb.model.navigation     import NavigationTree
from waskaweb.lib.security         import checkRole
from waskaweb.model.document       import listDocuments
from waskaweb.model.data           import FilteredNodeComponentProxy
from waskaweb.model.logbook        import Logbook, LogbookEntry, LogbookEntryChecker

from waskaweb.model.phases_factory   import phases_pairs
from waskaweb.model.phases           import Phase
from waskaweb.model.phase_transition import symbol_phase, phase_neighbors

from waskaweb.lib.db        import db
from waskaweb.lib.xmlimport import importFromXML

from waskaweb.controllers.appointment import DELETE_APPOINT_NOTIFICATION_SUCCESS, \
                                             DELETE_APPOINT_NOTIFICATION_FAILED, \
                                             DELETE_APPOINT_NOTIFICATION_TEXT_SUCCESS, \
                                             DELETE_APPOINT_NOTIFICATION_TEXT_FAILED, \
                                             DELETE_APPOINT_CONFIRM, \
                                             DELETE_APPOINT_CONFIRM_TEXT, \
                                             CREATE_APPOINT_NOTIFICATION_SUCCESS, \
                                             CREATE_APPOINT_NOTIFICATION_TEXT_SUCCESS, \
                                             EDIT_APPOINT_NOTIFICATION_SUCCESS, \
                                             EDIT_APPOINT_NOTIFICATION_TEXT_SUCCESS

log = logging.getLogger(__name__)


PDF_DUB_NOTIFICATION      = u"Die Fallakte wurde nicht importiert."
PDF_DUB_NOTIFICATION_TEXT = \
u"""Eine Fallakte mit der Kundennummer %s existiert bereits."""

PDF_IMPORT_FAILED_NOTIFICATION_TEXT = \
u"""Die angegebene Datei konnte nicht als PDF-Fallakte erkannt werden."""

PRINTALL_NOTIFICATION = u'Gesamte Fallakte drucken'
PRINTALL_NOTIFICATION_TEXT = u'Leider ist die von Ihnen aufgerufene Funktion in der BETA Version dieser Anwendung noch nicht enthalten.' 

DELETE_CONFIRM = u"""Fallakte löschen?""" 
DELETE_CONFIRM_TEXT = u"""Wollen Sie wirklich die Fallakte löschen und die Daten unwiederbringlich verlieren?""" 
MARKDELETE_CONFIRM_TEXT = u"""Wollen Sie wirklich die Fallakte zum Löschen freigeben? Die Fallakte steht Ihnen danach nicht mehr zur Bearbeitung zur Verfügung und wird der Administration zum Löschen vorgelegt.""" 

ANONYMIZE_CONFIRM = u"""Fallakte anonymisieren?"""
ANONYMIZE_CONFIRM_TEXT = u"""Wollen Sie wirklich die Fallakte anonymisieren und die personenbezogenen Daten unwiederbringlich verlieren?. Die Fallakte steht danach nicht mehr zur Bearbeitung zur Verfügung. Die anonymisierte Fallakte wird weiterhin in der Auswertung berücksichtigt werden."""
MARKANONYMIZE_CONFIRM_TEXT = u"""Wollen Sie wirklich die Fallakte zum Anonymisieren freigeben? Die Fallakte steht Ihnen danach nicht mehr zur Bearbeitung zur Verfügung und wird der Administration zur Anonymisierung vorgelegt.""" 

ANONYMIZE_SUCCESS = u"""Fallakte anonymisiert!"""
ANONYMIZE_SUCCESS_TEXT = u"""Die personenbezogenen Daten der Fallakte wurden gelöscht und die Fallakte aus der Übersicht entfernt. Bitte klicken Sie auf "OK", um zur Fallaktenübersicht fortzufahren."""
SET_PHASE_NOTIFICATION_SUCCESS = u"""Phase gewechselt!"""
SET_PHASE_NOTIFICATION_TEXT_SUCCESS= u"""Die Phase konnte erfolgreich gewechselt werden. Bitte folgen Sie den aufgeführten Schritten, um zur Phasenübersicht fortzufahren."""
MARKANONYMIZE_SUCCESS = u"""Fallakte wurde zur Anonymisierung freigegeben!"""
MARKANONYMIZE_SUCCESS_TEXT = u"""Die Fallakten wurde zur Anonymisierung freigeben und liegt der Administration zur endgültigen Anonymisierung vor. Bitte klicken Sie auf "OK", um zur Fallaktenübersicht fortzufahren."""

MARKANONYMIZE_INCOMPLETE_PHASE_FAILED = u"""Anonymisierung nicht möglich."""
MARKANONYMIZE_INCOMPLETE_PHASE_FAILED_TEXT = u"""Eine Anonymisierung ist für dieser Fallakte derzeit nicht möglich. Bitte prüfen Sie, ob Sie die aktuell laufende Phase beendet haben, und ob alle nötigen Pflichtfelder für die aktuelle oder zurückliegende Phasen ausgefüllt wurden. Erfüllen Sie die genannten Anforderungen zur Anonymisierung und Wiederholen Sie den Vorgang. Klicken Sie auf "OK", um fortzufahren."""
IMPORT_FAILED = u"""Fallakte wurde nicht importiert!"""
IMPORT_FAILED_TEXT = u"""Bei dem Importieren der Datei ist ein Fehler aufgetreten. Es konnten keine Daten der Fallakte importiert werden. Klicken Sie auf "OK", um zum Import-Dialog fortzufahren."""

IMPORTS_FAILED = u"""Fallakten wurden nicht importiert!"""
IMPORTS_FAILED_TEXT = u"""Bei dem Importieren der Datei sind Fehler aufgetreten. Es konnten ggf. nur einzelne Fallakten importiert werden. Klicken Sie auf "OK", um zum Import-Dialog fortzufahren."""

IMPORT_XML_SUCCESS = u"""Fallakten erfolgreich importiert!"""
IMPORT_XML_SUCCESS_TEXT = u"""Die Fallakten wurde erfolgreich importiert. Bitte klicken Sie auf "OK", um zur Fallaktenübersicht fortzufahren."""

IMPORT_ANON_SUCCESS = u"""Fallakte importiert und zur Anonymisierung freigegeben!"""
IMPORT_ANON_SUCCESS_TEXT = u"""Die Fallakte wurde erfolgreich importiert und zur Anonymisierung freigegeben. Die Fallakte liegt der Administration zur endgültigen Anonymisierung vor. Bitte klicken Sie auf "OK", um zur Fallaktenübersicht fortzufahren."""

RESTORE_CONFIRM = u"""Fallakte wiederherstellen?"""
RESTORE_CONFIRM_TEXT = u"""Wollen Sie wirklich die Fallakte wiederherstellen? Die Fallakte ist danach wieder für die Nutzer bearbeitbar."""

CLOSE_CONFIRM = u"""Fallakte schließen?"""
CLOSE_CONFIRM_TEXT = u"""Wollen Sie wirklich die Fallakte schließen?"""

OPEN_CONFIRM = u"""Geschlossene Fallakte öffnen?"""
OPEN_CONFIRM_TEXT = u"""Wollen Sie wirklich die Fallakte wieder öffnen?"""

LOGBOOK_ENTRY_CREATE_FAILED = u"Tagebucheintrag wurde nicht angelegt!"
LOGBOOK_ENTRY_CREATE_FAILED_TEXT = u"Beim Anlegen eines Tagebucheintrages is ein Fehler aufgetreten."

CREATE_LOGBOOK_ENTRY_SUCCESS = u"Eintrag ins Tagebuch erfolgreich."
CREATE_LOGBOOK_ENTRY_SUCCESS_TEXT = u"Der Tagebucheintrag wurde erfolgreich durchgeführt."

DELETE_LOGBOOK_ENTRY_CONFIRM = u"Tagebucheintrag löschen?"
DELETE_LOGBOOK_ENTRY_CONFIRM_TEXT = u"""Wollen Sie diesen Tagebucheintrag wirklich löschen?
Er ginge damit unwiederbringlich verloren."""

LOGBOOK_ENTRY_DELETE_SUCCESS = u"Tagebucheintrag erfolgreich gelöscht!"
LOGBOOK_ENTRY_DELETE_SUCCESS_TEXT = u"Der Tagebucheintrag wurde erfolgreich gelöscht!"

LOGBOOK_ENTRY_DELETE_FAILED = u"Löschen des Tagebucheintrags fehlgeschlagen!"
LOGBOOK_ENTRY_DELETE_FAILED_TEXT = u"Beim Löschen des Tagebucheintrags ist ein Fehler aufgetreten."

LOGBOOK_ENTRY_SHOW_FAILED = u"Anzeigen des Tagebucheintrags fehlgeschlagen!"
LOGBOOK_ENTRY_SHOW_FAILED_TEXT = u"Beim Anzeigen des Tagebucheintrags ist ein Fehler aufgetreten."

LOGBOOK_ENTRY_SAVE_SUCCESS = u"Änderung des Eintrages erfolgreich!"
LOGBOOK_ENTRY_SAVE_SUCCESS_TEXT = u"Der Eintrag im Tagebuch wurde erfolgreich geändert."

LOGBOOK_ENTRY_SAVE_FAILED = u"Änderung des Eintrages fehlgeschlagen!"
LOGBOOK_ENTRY_SAVE_FAILED_TEXT = u"Die Änderung des Eintrages im Tagebuch ist fehlgeschlagen."

def load_case(id):
        '''Return a case object'''
        factory   = CaseFactory()
        case = factory.loadById(id)

        # New session code
        session_case = case.getSessionCase()
        session['case'] = session_case

        navigation = NavigationTree(case)
        session['navigation.tree'] = navigation
        session.save()
        return case

def checkPrivacyStatement(context='select'):
    '''Decorator for the methods in the case controller'''
    def validate(func, self, *args, **kwargs):
        #load case and check if the statement is signed
        if context == "showAppointment":
            try:
                app_id = int(args[0])
                confirmed = int(args[1])
            except:
                raise HTTPNotFound()
            app_factory = CaseAppointmentFactory()
            app = app_factory.loadById(app_id)
            case_id = app.case_id
            c.targeturl = h.url_for(controller="/case", \
                                    action="showAppointment", \
                                    id=app_id, \
                                    confirmed=1)
        else:
            try:
                case_id = int(args[0])
                confirmed = int(args[1])
            except:
                raise HTTPNotFound()
            c.targeturl = h.url_for(controller="/case", \
                                    action=context, \
                                    id=case_id, \
                                    confirmed=1)

        try:
            case = load_case(case_id)
        except:
            return render('/tests/trace.mako')
            self.showError()
        statement = case.getPrivacyStatement()
        # Check if there is a "Einverstaendniserklaerung" None if it is not either signed or declined
        if (not statement.isSigned() is None) or (confirmed == 1):
            return func(self, *args, **kwargs)
        else:
            c.ds_id = case_id
            return render('/statement/dialogs/missing_statement.mako')
    return decorator(validate)

class CaseController(CasebaseController):

    def __before__(self):
        CasebaseController.__before__(self)

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def index(self):
        redirect_to(controller="/case_overview")

    def __init_case(self, id):
        '''Return a case object'''
        case = load_case(id)
        self.navigation = session.get('navigation.tree')
        return case

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    @checkPrivacyStatement()
    def select(self, id, confirmed=0):
        try:
            case = self.__init_case(id)
        except StandardError, err:
            self.showError()
        return self.digest(id)

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def digest(self, id):
        id = self._checkInt(id)
        case = self._loadCase(id)
        c.digest = case.getDigest()
        c.ds_id = id
        c.last_access = case.getState().getAccessTime('%d.%m.%Y %H:%M')
        c.daysToMaxSavetime = case.getState().daysToMaxSavetime()
        c.maxsavetime = case.getState().getMaxSavetime()
        c.finished = case.getState().isCmFinished()
        c.canceled = case.getState().isCmCanceled()
        #ti = self.getNavigation().getTreeItem('0')
        #if not ti: return '404'
        c.form_navigation = self._getFormNavigation()
        c.print_form = "digest" # Which page to open in popup on printing?
        return render('/casemanagement/digest.mako')

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def printout(self, ds_id, page_id, form=None):
        ds_id   = self._checkInt(ds_id)
        page_id = self._checkInt(page_id)
        c.print_version = 1 
        if form == "digest":
            return self.digest(ds_id) 
        elif form == "show":
            return self.show(ds_id, page_id)
        else:
            raise HTTPNotFound()

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def printAll(self, id, empty):
        id = self._checkInt(id)
        empty = self._checkBool(empty)
        case = self._loadCase(id)
        formular = []
        c.print_version = 1
        # First page is print version of the digest ()
        c.digest = case.getDigest(empty)
        digest = render('/casemanagement/digest_body.mako')
        formular.append(unicode(digest, 'utf-8'))
        # append pagebreak
        formular.append('<span style="page-break-after:always"></span>')
        for ti in self.getNavigation().getAllItems():
            if ti.key in ['27', '28', '29', '30']: continue #ignoring empty aid pages
            content, navigation = self._getRenderdFormElements(ti, id, ro_mode=True, no_data=empty)
            formular.append("""<h1>%s</h1>""" % ti.description)
            if not ti.children or ti.key == '14': #Only add leaves
                formular.append(content)
            if ti.key == '14': # found aid-overview
                aid_list = AidList(id)
                for aid in aid_list.getDatasets():
                    ti_rg   = self.getNavigation().getTreeItem(aid.page_id)
                    content, navigation = self._getRenderdFormElements(ti_rg, aid.id, ro_mode=True, no_data=empty)
                    formular.append("""<h1>%s</h1>""" % ti_rg.description)
                    formular.append(content)

        c.formular = "<br>".join(formular)
        page = render('/casemanagement/formular.mako')
        return page

    @checkRole('cm_ka')
    def new(self):
        session_case = SessionCase()
        session['case'] = session_case
        session.save()
        redirect_to('new2')

    @checkRole('cm_ka')
    def new2(self):
        c.case = session.get('case')
        return render('/casemanagement/new.mako')

    @checkRole('cm_ka')
    def newAction(self):
        # TODO: Use htmlfill here!
        try:
            session_case = session['case']
        except KeyError:
            session_case = SessionCase()

        session_case.setData(request.params)
        session.save()

        # Do we want to print the statement?
        if request.params.has_key('print'):
            c.print_version = 1
            statement = PrivacyStatement()
            agency = Agency()
            c.content = statement.fillout(session_case, agency) 
            return render('statement/default_statement.mako')

        # Or do we want to store the new case?
        elif request.params.has_key('save'):
            validator = NewCaseForm()
            try:
                form_result = validator.to_python(request.params)
                # 1. Create a new case
                factory = CaseFactory()
                case = factory.createNew()
                statement = case.getPrivacyStatement()

                # 2. Do we want to sign the statement?
                if form_result.has_key('statement_signed'):
                    statement.sign()

                self.__init_case(case.id)
                # 3. Get Page from case and store the data 
                # TODO: Enable this is the future

                # This filter fixes issue45. The proxy returns only elements
                # which are in the params. This way only those values get
                # modified and no other values will be set to NULL.
                filter = FilteredNodeComponentProxy(request.params.dict_of_lists())

                document = g.formedTree
                page = document.getPageStore("page_1", proxy=filter)
                page.setId(case.id)

                new_errors = page.save(
                    request.params.dict_of_lists(),
                    {}, document)

                session_case.setFormErrors(new_errors)
                session.save()

                # Prepare params so that _save will eat them an gets all needed
                # values
                new_params = request.params.dict_of_lists()
                new_params['ds'] = [case.id]

                load_case(case.id)

            except formencode.Invalid, error:
                c.form_result = error.value
                c.form_errors = error.error_dict or {}
                return render('/casemanagement/new.mako')

            if statement.isSigned():
                return self.edit(str(case.id), '0')
            else:
                # Render Nagscreen to inform the user that he is about to open a
                # case which does not have a signed statement
                c.ds_id = case.id
                c.targeturl = h.url_for(controller="/case", action="select", id=c.ds_id, confirmed=1)
                return render('statement/dialogs/missing_statement.mako')

    @checkRole('cm_ka')
    def edit(self, ds_id, page_id):
        ds_id        = self._checkInt(ds_id)
        page_id      = self._checkInt(page_id)
        ti           = self.getNavigation().getTreeItem(page_id)
        c.ds_id      = ds_id
        real_id      = ti.realId(ds_id)
        # TODO: Move repieatgroup handling in an own controller
        #factory     = CaseFactory()
        #c.case      = factory.loadById(ds_id)
        case_session = session.get('case')
        case_session.setPageId(page_id)
        case_session.setMode("edit")
        case_session.setPageId(page_id)
        session.save()

        c.page_id = page_id
        c.print_form = "show" # Which page to open in popup on printing?
        content, navigation = self._getRenderdFormElements(ti, ds_id, page_id, ro_mode=False)
        c.formular = content
        c.form_navigation = navigation 
        return render('/casemanagement/formular.mako')

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def show(self, ds_id, page_id):
        ds_id        = self._checkInt(ds_id)
        page_id      = self._checkInt(page_id)
        ti           = self.getNavigation().getTreeItem(page_id)
        c.ds_id      = ds_id
        real_id      = ti.realId(ds_id)
        c.page_id    = page_id
        c.print_form = "show" # Which page to open in popup on printing?
        case_session = session.get('case')
        case_session.setPageId(page_id)
        case_session.setMode("show")
        session.save()
        content, navigation = self._getRenderdFormElements(ti, ds_id, page_id, ro_mode=True)
        c.formular = content
        c.form_navigation = navigation 
        return render('/casemanagement/formular.mako')

    @checkRole(('admin_ka', 'cm_ka', 'pb_ka'))
    def showHelp(self, help_id):
        help = g.helpData.getHelp(help_id)
        if help is None: raise HTTPNotFound()
        c.helpdata  = unicode(help, 'utf-8')
        return render('/casemanagement/help.mako')

    @checkRole(('admin_ka', 'cm_ka', 'pb_ka'))
    def showRequired(self, help_id):
        required = g.helpData.getAnnotations(help_id)
        if not required: raise HTTPNotFound()
        c.required  = unicode(required, 'utf-8')
        return render('/casemanagement/required.mako')

    @checkRole(('admin_ka', 'cm_ka', 'pb_ka'))
    def showHelpEvaluation(self, help_id):
        required = g.helpData.getAnnotations(help_id, ('evaluation',))
        if not required: raise HTTPNotFound()
        c.required  = unicode(required, 'utf-8')
        return render('/casemanagement/required.mako')

    @checkRole('cm_ka')
    def save(self):
        case_session = session.get('case')
        try:
            ti = self.getNavigation().getTreeItem(request.params["page"])
        except:
            ti = None

        if ti is None:
            redirect_to(controller="/case_overview")

        # If we edit page_1 we better update the infofield because the names
        # may have been edited 
        if ti.page == "page_1":
            try:
                case_session.last_name = request.params.getone("name_1")
                case_session.first_name = request.params.getone("vorname_1")
                session.save()
            except KeyError:
                print >> sys.stderr, "Error! Could not update Infofield"

        # Get old formerrors, save data and store new formdata
        old_errors = case_session.getFormErrors()
        ds_id = request.params["ds"]
        new_errors, page = self.__save(
            ti.page, 
            request.params.dict_of_lists(), 
            old_errors, 
            ti.realId(ds_id))

        if new_errors:
            c.status_msg = h.status_message("Eingabedaten fehlerhaft! Daten wurden nicht gespeichert.")
        else:
            c.status_msg = h.status_message(u'Daten erfolgreich gespeichert. Bitte beachten Sie die <a href="/case/showhelp/datenschutz_veraenderte_daten" target="_blank">Hinweise zur Datenschutz.</a>')

        if ti.page == "page_0":
            try:
                case_session.first_meeting = page.getData("erstgespraech")
                case_session.knr           = page.getData("kundennummer")
            except:
                traceback.print_exc(file=sys.stderr)
            session.save()

        case_session.setFormErrors(new_errors)
        session.save()

        # Only proceed to next page if the page does not contain errors
        if request.params.get('__formular_next') and len(h.getFormularErrors(ti.page)) <= 0:

            # XXX: This ultra ugly!
            if int(ti.key) in [27,28,29,30]:
                try:
                    nds = int(case_session.id)
                except:
                    print >> sys.stderr, "Case id not found"
                    nds = None
                if not nds is None:
                    ti.showPath()
                    ds_id   = ti.realId(int(nds))
                    page_id = "14"
                    return self.edit(ds_id, page_id) 

            nkey = ti.nextKey()
            if not nkey is None:
                nti = self.getNavigation().getTreeItem(nkey)
                if not nti is None:
                    ti = nti
                    ti.showPath()
        return self.edit(ds_id, ti.key) 

    def __save(self, pageName, params, old_errors, ds_id):
        # fetch dataset if valid
        document = g.formedTree
        page = document.getPageStore(pageName)
        page.setId(ds_id)
        result = page.save(params, old_errors, document)
        return result, page

    @checkRole('cm_ka')
    def importCase(self):
        return render('/casemanagement/upload.mako')

    @checkRole('cm_ka')
    def importXMLAction(self):
        factory = CaseFactory()
        try:
            xml = request.POST['xmlfile']
            conn = None
            try:
                conn = db.getConnection()
                importFromXML(xml.file, conn)
            finally:
                db.recycleConnection(conn)

            xml.file.close()
            c.success_for  = IMPORT_XML_SUCCESS
            c.success_text = IMPORT_XML_SUCCESS_TEXT
            c.url_ok       = h.url_for(controller="/case_overview")
            return render('/casemanagement/dialogs/success_xml_import.mako')
        except:
            c.failed_for  = IMPORTS_FAILED
            c.failed_text = IMPORTS_FAILED_TEXT
            c.url_ok = "/case/importCase"
            return render('/casemanagement/dialogs/failed_xml_import.mako')

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def export(self):
        return render('/casemanagement/export.mako')

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def createExportScheme(self):
        return render('casemanagement/create_export_scheme.mako')

    @checkRole(('admin_ka', 'cm_ka', 'pt_dlr'))
    def evaluate(self):
        return render('casemanagement/evaluation.mako')

    @checkRole(('admin_ka', 'cm_ka'))
    def organisation(self, id):
        id        = self._checkInt(id)
        ulist     = UserListObject()
        list      = UserGroupList()
        c.case    = self._loadCase(id)
        c.standin = c.case.getStandin()
        c.editor  = c.case.getEditor()
        c.current_user = session['USER_AUTHORIZED']

        # group list for standin
        filter = [u.gid for u in ulist.getAdminList()]
        filter.append(c.editor.id)
        filter2 = filter[:]
        filter.append(c.current_user.gid)
        c.group_list  = list.getGroupList(filter)
        c.group_list2 = list.getGroupList(filter2)

        # userlist for editor
        filter3 = [u.id for u in ulist.getAdminList()]
        filter3.append(c.current_user.id)
        filter3.append(c.editor.id)
        c.user_list  = ulist.getUserList(filter3)

        c.ds_id = id
        c.form_navigation = self._getFormNavigation()
        return render('/casemanagement/organisation.mako')

    @checkRole('admin_ka')
    def setEditorAction(self):
        validator = SetEditorForm()
        try:
            form_result = validator.to_python(request.params)
            case = self._loadCase(form_result.get('case_id'))
            case.setEditor(form_result.get('editor'))
            c.status_msg = h.status_message("Bearbeiter erfolgreich gesetzt.")
            return self.organisation(form_result.get('case_id'))
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            c.status_msg = h.status_message("Fehler! Bearbeiter nicht gesetzt.")
            return render('/casemanagement/organisation.mako')

    @checkRole(('admin_ka', 'cm_ka'))
    def setStandinAction(self):
        validator = SetStandinForm()
        data = formencode.variabledecode.variable_decode(request.params)
        try:
            form_result = validator.to_python(data)
            case = self._loadCase(form_result.get('case_id'))
            standin = case.getStandin()
            standin.setGroups(form_result.get('standin'))
            c.status_msg = h.status_message("Vertretung erfolgreich gesetzt.")
            return self.organisation(form_result.get('case_id'))
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            c.status_msg = h.status_message("Fehler! Vertretung nicht gesetzt.")
            return render('/casemanagement/organisation.mako')

    @checkRole(('admin_ka', 'cm_ka'))
    def appointments(self, id):
        id                 = self._checkInt(id)
        case               = self._loadCase(id)
        c.ds_id            = id
        c.form_navigation  = self._getFormNavigation()
        c.remindlist       = CaseReminderOverview(id)
        c.appointmentlist  = CaseAppointmentOverview(id)
        return render('/casemanagement/appointments.mako')

    @checkRole(('admin_ka', 'cm_ka'))
    def printLogbook(self, id):
        id                 = self._checkInt(id)
        case               = self._loadCase(id)
        c.ds_id            = id
        c.form_navigation  = self._getFormNavigation()
        c.print_version = 1 
        logbook = Logbook()
        logbook.loadById(id)
        c.logbook = logbook

        # Evaluation
        # c.eval_effort = logbook.getEvaluation()
        return render('/logbook/overview_print.mako')

    @checkRole(('admin_ka', 'cm_ka'))
    def logbook(self, id, sort_field='datum', sort_order='desc'):
        id                 = self._checkInt(id)
        case               = self._loadCase(id)
        c.ds_id            = id
        c.form_navigation  = self._getFormNavigation()
        logbook = Logbook()
        logbook.loadById(id, sort_field, sort_order)
        c.logbook = logbook

        # Evaluation
        c.eval_effort = logbook.getEvaluation()
        return render('/logbook/overview.mako')

    @checkRole(('admin_ka', 'cm_ka'))
    def phase(self, id):
        id     = self._checkInt(id)
        case   = self._loadCase(id)
        fields = case.getRequiredFields()

        phase = case.getState().getPhase()

        session_case = session.get('case')

        mode = session_case and session_case.getMode() or "show" 
        link = lambda ti: '"/case/%s/%d/%s"' % (mode, ti.realId(id), ti.key)

        c.phase_pairs   = phases_pairs(fields, phase, self.getNavigation(), link)
        c.current_phase = Phase(phase, True)

        c.form_navigation  = self._getFormNavigation()
        return render('/casemanagement/phase.mako')

    @checkRole(('cm_ka'))
    def setPhaseAction(self):
        validator = SetPhaseForm()
        factory = CaseFactory()
        c.form_errors = {}
        c.form_result = {}
        try:
            try:
                form_result = validator.to_python(request.params)
                # Load case and do something on phase 
                case = factory.loadById(form_result.get('case_id'))

                new_phase, new_phase_symbol = None, None

                for new_phase_symbol in request.params.iterkeys():
                    try:
                        new_phase = symbol_phase(new_phase_symbol)
                        break
                    except KeyError:
                        pass

                if new_phase is None:
                    raise StandardError("Phase nicht gefunden.")

                state = case.getState()

                current_phase = state.getPhase()

                #if not phase_neighbors(current_phase, new_phase):
                #    raise StandardError("Phasen sind nicht benachbart.")

                case_session = session.get('case')

                if case_session.isYoungerThan(INCONSISTENCY_CHECK_AFTER):
                    fields = case.getRequiredFields()
                    if not fields.isPhaseComplete(new_phase_symbol):
                        raise StandardError("Phase ist nicht komplett.")

                state.setPhase(new_phase)

                case_session.state = state
                session.save()

                # After setting the phase. Show print-dialog
                c.success_for   = SET_PHASE_NOTIFICATION_SUCCESS
                c.success_text  = SET_PHASE_NOTIFICATION_TEXT_SUCCESS
                c.url_ok        = h.url_for(controller='/case', \
                                            action='phase', \
                                            id=case.id)

                #c.form_navigation = self._getFormNavigation()
                return render('/casemanagement/dialogs/success_set_phase.mako')
            except formencode.Invalid, error:
                c.form_result = error.value
                c.form_errors = error.error_dict or {}
                return self.phase(form_result.get('case_id'))
        except:
            return render('/tests/trace.mako')

    @checkRole('cm_ka')
    def newLogbookEntry(self, id):
        entry = LogbookEntry()
        entry.setDate(datetime.now())
        entry.setKind(-1)
        form_defaults = {}
        form_defaults['date']  = entry.getDate()
        form_defaults['time']  = entry.getTime()
        form_defaults['kind']  = entry.getKindAsInt()
        id                = self._checkInt(id)
        c.ds_id           = id
        c.entry           = entry
        c.form_navigation = self._getFormNavigation()
        c.form_errors     = {}
        form = render('/logbook/new.mako')
        return formencode.htmlfill.render(unicode(form, 'utf-8'), \
            defaults=form_defaults, \
            auto_insert_errors=False, \
            errors=c.form_errors)

    @checkRole('cm_ka')
    @checkPrivacyStatement('keepActive')
    def keepActive(self, id, confirmed):
        entry = LogbookEntry()
        entry.setDate(datetime.now())
        entry.setKind(-2)
        form_defaults = {}
        form_defaults['date']  = entry.getDate()
        form_defaults['time']  = entry.getTime()
        form_defaults['kind']  = entry.getKindAsInt()
        form_defaults['kind']  = entry.getKindAsInt()
        form_defaults['short_notice']  = u"Aufbewahrung verlängert" 
        form_defaults['notice']  = u"Bitte geben Sie hier den Grund für die Verlängerung der Aufbewahrungsfrist ein." 
        id                = self._checkInt(id)
        c.ds_id           = id
        c.entry           = entry
        c.form_navigation = self._getFormNavigation()
        c.form_errors     = {}
        form = render('/logbook/new.mako')
        return formencode.htmlfill.render(unicode(form, 'utf-8'), \
            defaults=form_defaults, \
            auto_insert_errors=False, \
            errors=c.form_errors)

    @checkRole('cm_ka')
    def createNewLogbookEntry(self):
        errors = []
        checker = LogbookEntryChecker(request.params)
        validator = CreateLogbookForm()
        c.form_errors = {}
        c.form_result = {}

        try:
            c.ds_id           = session['case'].id
            c.form_navigation = self._getFormNavigation()
            form_result = validator.to_python(request.params)
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            form = render('/logbook/new.mako')
            return formencode.htmlfill.render(unicode(form, 'utf-8'), \
                defaults=c.form_result, \
                auto_insert_errors=False, \
                errors=c.form_errors)

        #if checker.hasErrors():
        #    c.ds_id           = session['case'].id
        #    c.entry           = checker.getLogbookEntry()
        #    c.form_navigation = self._getFormNavigation()
        #    c.form_errors     = checker.getErrors()

        try:
            logbook = Logbook(checker.getCaseId())
            logbook.addEntry(checker.getLogbookEntry())
            logbook.store(session['USER_AUTHORIZED'].id)
            c.url_ok = "/case/logbook/%s" % str(session['case'].id)
            c.success_for   = CREATE_LOGBOOK_ENTRY_SUCCESS 
            c.success_text  = CREATE_LOGBOOK_ENTRY_SUCCESS_TEXT
            return render('/logbook/dialogs/success_create_logbook_entry.mako')
        except:
            traceback.print_exc(file=sys.stderr)
            c.failed_for  = LOGBOOK_ENTRY_CREATE_FAILED
            c.failed_text = LOGBOOK_ENTRY_CREATE_FAILED_TEXT
            c.url_ok = "/case/logbook/%s" % str(session['case'].id)
            return render('/logbook/dialogs/failed_create_logbook_entry.mako')

    @checkRole(('cm_ka', 'admin_ka'))
    def showLogbookEntry(self, id):
        id = self._checkInt(id)
        try:
            entry = LogbookEntry()
            entry.loadById(id)
        except:
            c.failed_for  = LOGBOOK_ENTRY_SHOW_FAILED
            c.failed_text = LOGBOOK_ENTRY_SHOW_FAILED_TEXT
            c.url_ok = "/case/logbook/%s" % str(session['case'].id)
            traceback.print_exc(file=sys.stderr)
            return render('/logbook/dialogs/failed_show_logbook_entry.mako')

        c.entry           = entry
        c.form_navigation = self._getFormNavigation()
        return render('/logbook/show.mako')

    @checkRole('cm_ka')
    def editLogbookEntry(self, id):
        id = self._checkInt(id)
        c.form_navigation = self._getFormNavigation()
        c.form_errors = {}
        entry_data = {}
        try:
            entry = LogbookEntry()
            entry.loadById(id)
            entry_data['id'] = entry.id
            entry_data['date'] = entry.getDate()
            entry_data['time'] = entry.getTime()
            entry_data['duration'] = entry.getDuration()
            entry_data['kind'] = entry.getKindAsInt()
            entry_data['notice'] = entry.getNotice()
            entry_data['short_notice'] = entry.getShortNotice()
            c.entry = entry
            form = render('/logbook/edit.mako')
            return formencode.htmlfill.render(unicode(form, 'utf-8'), \
                defaults=entry_data, \
                auto_insert_errors=False, \
                errors={})
        except:
            traceback.print_exc(file=sys.stderr)
            c.failed_for  = LOGBOOK_ENTRY_SHOW_FAILED
            c.failed_text = LOGBOOK_ENTRY_SHOW_FAILED_TEXT
            c.url_ok = "/case/logbook/%s" % str(session['case'].id)
            return render('/logbook/dialogs/failed_show_logbook_entry.mako')

    @checkRole('cm_ka')
    def saveLogbookEntry(self):
        params = request.params
        id = self._checkInt(params.getone('id'))
        c.form_navigation = self._getFormNavigation()
        checker = LogbookEntryChecker(params, check_case_id=False)
        validator = EditLogbookForm()

        try:
            c.ds_id = session['case'].id
            form_result = validator.to_python(params)
        except formencode.Invalid, error:
            c.form_result = error.value
            c.form_errors = error.error_dict or {}
            form = render('/logbook/new.mako')
            return formencode.htmlfill.render(unicode(form, 'utf-8'), \
                defaults=c.form_result, \
                auto_insert_errors=False, \
                errors=c.form_errors)

        #if checker.hasErrors():
        #    c.form_navigation = self._getFormNavigation()
        #    c.errors = checker.getErrors()
        #    c.entry  = checker.getLogbookEntry()
        #    c.entry.setId(id)
        #    return render('/logbook/edit.mako')

        try:
            entry = checker.getLogbookEntry()
            entry.setId(id)
            entry.storeForUser(session['USER_AUTHORIZED'].id)
            c.success_for  = LOGBOOK_ENTRY_SAVE_SUCCESS 
            c.success_text = LOGBOOK_ENTRY_SAVE_SUCCESS_TEXT
            c.url_ok       = "/case/logbook/%s" % str(session['case'].id)
            return render('/logbook/dialogs/success_save_logbook_entry.mako')
        except:
            traceback.print_exc(file=sys.stderr)
            c.failed_for  = LOGBOOK_ENTRY_SAVE_FAILED
            c.failed_text = LOGBOOK_ENTRY_SAVE_FAILED_TEXT
            c.url_ok      = "/case/logbook/%s" % str(session['case'].id)
            return render('/logbook/dialogs/failed_save_logbook_entry.mako')

    @checkRole('cm_ka')
    def deleteLogbookEntry(self, id, confirmed=False):
        id        = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        c.form_navigation = self._getFormNavigation()
        if confirmed == 1:
            try:
                Logbook().deleteEntryById(int(id))
                c.success_for  = LOGBOOK_ENTRY_DELETE_SUCCESS 
                c.success_text = LOGBOOK_ENTRY_DELETE_SUCCESS_TEXT
                c.url_ok       = "/case/logbook/%s" % str(session['case'].id)
                return render('/logbook/dialogs/success_delete_logbook_entry.mako')
            except:
                traceback.print_exc(file=sys.stderr)
                c.failed_for  = LOGBOOK_ENTRY_DELETE_FAILED
                c.failed_text = LOGBOOK_ENTRY_DELETE_FAILED_TEXT
                c.url_ok = "/case/logbook/%s" % str(session['case'].id)
                return render('/logbook/dialogs/failed_delete_logbook_entry.mako')
        else:
            c.confirm_for = DELETE_LOGBOOK_ENTRY_CONFIRM
            c.question    = DELETE_LOGBOOK_ENTRY_CONFIRM_TEXT
            c.url_yes     = "/case/deleteLogbookEntry/%s/1" % id
            c.url_no      = "/case/logbook/%s" % session['case'].id
            return render('/logbook/dialogs/confirm_delete_logbook_entry.mako')

    @checkRole(('admin_ka', 'cm_ka'))
    @checkPrivacyStatement('showAppointment')
    def showAppointment(self, id, confirmed):
        factory           = CaseAppointmentFactory()
        id                = self._checkInt(id)
        app               = factory.loadById(id)
        c.appointment     = app
        c.form_navigation = self._getFormNavigation()
        return render('/casemanagement/showAppointment.mako')

    @checkRole(('cm_ka'))
    def editAppointment(self, id):
        id = self._checkInt(id)
        factory = CaseAppointmentFactory()
        app = factory.loadById(id)

        c.form_result = {'start_date'  : dd_mm_YYYY(app.start_date), \
                         'start_time'  : HH_MM(app.start_date), \
                         'end_date'    : dd_mm_YYYY(app.end_date), \
                         'end_time'    : HH_MM(app.end_date), \
                         'title'       : app.title, \
                         'id'          : app.id, \
                         'case_id'     : app.case_id, \
                         'type'        : app.type, \
                         'description' : app.description}

        c.form_errors = {}
        c.appointment = app
        c.form_navigation = self._getFormNavigation()
        form = render('/casemanagement/editAppointment.mako')
        return formencode.htmlfill.render(unicode(form, 'utf-8'), \
                defaults=c.form_result, \
                auto_insert_errors=False, \
                errors=c.form_errors)

    @checkRole(('cm_ka'))
    def editAppointmentAction(self):
        validator = CreateAppointmentForm()
        factory = CaseAppointmentFactory()
        c.form_errors = {}
        c.form_result = {}
        try:
            try:
                form_result = validator.to_python(request.params)
                # Load appointment and set data
                appointment = factory.loadById(form_result.get('id'))
                appointment.setData(form_result)
                appointment.store()
                c.success_for   = EDIT_APPOINT_NOTIFICATION_SUCCESS
                c.success_text  = EDIT_APPOINT_NOTIFICATION_TEXT_SUCCESS
                c.url_ok        = h.url_for(controller='/case', \
                                            action='appointments', \
                                            id=appointment.case_id)

                c.form_navigation = self._getFormNavigation()
                return render('/casemanagement/dialogs/success_edit_appointment.mako')
            except formencode.Invalid, error:
                c.form_result = error.value
                c.form_errors = error.error_dict or {}
                c.appointment = factory.loadById(c.form_result.get('id'))
                form = render('/casemanagement/editAppointment.mako')
                return formencode.htmlfill.render(unicode(form, 'utf-8'), \
                            defaults=c.form_result, \
                            auto_insert_errors=False, \
                            errors=c.form_errors)
        except:
            return render('/tests/trace.mako')

    @checkRole(('cm_ka'))
    def deleteAppointment(self, id, confirmed=False):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        factory = CaseAppointmentFactory()
        appointment = factory.loadById(id)
        c.form_navigation = self._getFormNavigation()
        if confirmed == 1:
            if appointment.delete():
                c.success_for  = DELETE_APPOINT_NOTIFICATION_SUCCESS
                c.success_text = DELETE_APPOINT_NOTIFICATION_TEXT_SUCCESS
                c.url_ok       = "/case/appointments/%s" % appointment.case_id
                return render('/casemanagement/dialogs/success_delete_appointment.mako')
            else:
                c.failed_for  = DELETE_APPOINT_NOTIFICATION_FAILED
                c.failed_text = DELETE_APPOINT_NOTIFICATION_TEXT_FAILED
                c.url_ok       = "/case/appointments/%s" % appointment.case_id
                return render('/casemanagement/dialogs/failed_delete_appointment.mako')
        else:
            c.context     = "../main.mako" 
            c.confirm_for = DELETE_APPOINT_CONFIRM
            c.question    = DELETE_APPOINT_CONFIRM_TEXT
            c.url_yes     = "/case/deleteAppointment/%s/1" % id
            c.url_no      = "/case/appointments/%s" % appointment.case_id
            return render('/casemanagement/dialogs/confirm_delete_appointment.mako')

    @checkRole('cm_ka')
    def newAppointment(self, id):
        id                = self._checkInt(id)
        case              = self._loadCase(id)
        c.ds_id           = id
        c.form_navigation = self._getFormNavigation()
        c.form_errors = {}
        c.form_result = {}
        cd = datetime.now()
        c.form_defaults = {'start_date': dd_mm_YYYY(cd), \
                        'start_time': HH_MM(cd), \
                        'case_id': id, \
                        'type': 0}
        form = render('/casemanagement/newAppointment.mako')
        return formencode.htmlfill.render(unicode(form, 'utf-8'),defaults=c.form_defaults)

    @checkRole('cm_ka')
    def newReminder(self, id):
        id                = self._checkInt(id)
        case              = self._loadCase(id)
        c.ds_id           = id
        c.form_navigation = self._getFormNavigation()
        c.form_errors = {}
        c.form_result = {}
        cd = datetime.now()
        c.form_defaults = {'start_date': dd_mm_YYYY(cd), \
                        'start_time': HH_MM(cd), \
                        'case_id': id, \
                        'title': u"%s, %s" % (case.last_name, case.first_name), \
                        'description': u"Bitte geben Sie den Grund für die Wiedervorlage ein", \
                        'type': 1}
        form = render('/casemanagement/newAppointment.mako')
        return formencode.htmlfill.render(unicode(form, 'utf-8'),defaults=c.form_defaults)

    @checkRole(('cm_ka'))
    def newAppointmentAction(self):
        validator = CreateAppointmentForm()
        factory = CaseAppointmentFactory()
        c.form_errors = {}
        c.form_result = {}
        try:
            try:
                form_result = validator.to_python(request.params)
                # Create appointment
                appointment = factory.createNew(form_result.get('case_id'))
                appointment.setData(form_result)
                appointment.store()
                c.success_for   = CREATE_APPOINT_NOTIFICATION_SUCCESS
                c.success_text  = CREATE_APPOINT_NOTIFICATION_TEXT_SUCCESS
                c.url_ok        = h.url_for(controller='/case', \
                                            action='appointments', \
                                            id=appointment.case_id)
                c.form_navigation = self._getFormNavigation()

                return render('/casemanagement/dialogs/success_create_appointment.mako')
            except formencode.Invalid, error:
                c.form_result = error.value
                c.form_errors = error.error_dict or {}
                form = render('/casemanagement/newAppointment.mako')
                return formencode.htmlfill.render(unicode(form, 'utf-8'), \
                            defaults=c.form_result, \
                            auto_insert_errors=False, \
                            errors=c.form_errors)
        except:
            return render('/tests/trace.mako')

    @checkRole(('admin_ka', 'cm_ka'))
    def documents(self, id):
        id = self._checkInt(id)
        c.ds_id = id
        case = self._loadCase(id)
        c.form_navigation = self._getFormNavigation()
        c.files = case.getDocuments()
        return render('/documents/case_overview.mako')

    @checkRole('cm_ka')
    def newDocument(self, id):
        id                = self._checkInt(id)
        case              = self._loadCase(id)
        c.ds_id           = id
        c.form_navigation = self._getFormNavigation()
        return render('/documents/case_new.mako')

    @checkRole('cm_ka')
    def markForDelete(self, id, confirmed):
        id        = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        c.ds_id   = id
        if confirmed == 1:
            case = load_case(id)
            case.setState(3)
            c.success_for   =   u'Fallakte gelöscht!' 
            c.success_text  =   u'Die Fallakte wurde zum Löschen an die Administration weitergeleitet.' 
            c.url_ok        =   h.url_for(controller="/case_overview")
            return render('/casemanagement/dialogs/success_delete_cm.mako')
            redirect_to(controller="/case_overview")
        else:
            c.context = "../main.mako" 
            c.confirm_for = DELETE_CONFIRM
            c.question = MARKDELETE_CONFIRM_TEXT 
            c.url_yes = "/case/markForDelete/%s/1" % id
            c.url_no  = "/case_overview/"
            return render('/casemanagement/dialogs/confirm.mako')

    @checkRole('admin_ka')
    def delete(self, id, confirmed):
        id        = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        c.ds_id   = id
        if confirmed == 1:
            case = self._loadCase(id)
            if case.delete():
                c.success_for   =   u'Fallakte gelöscht!' 
                c.success_text  =   u'Bitte klicken Sie "OK", um fortzufahren.' 
                c.url_ok        =   h.url_for(controller="/case_overview")
                return render('/casemanagement/dialogs/success_delete.mako')
                #redirect_to(controller="/case_overview")
            # TODO: Create Errorpage
            return "Error! Could not delete ds."
        else:
            c.context = "../main.mako" 
            c.confirm_for = DELETE_CONFIRM 
            c.question = DELETE_CONFIRM_TEXT 
            c.url_yes = "/case/delete/%s/1" % id
            c.url_no  = "/case_overview/"
            return render('/casemanagement/dialogs/confirm_delete.mako')

    @checkRole('admin_ka')
    def restore(self, id, confirmed):
        id        = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        c.ds_id   = id
        if confirmed == 1:
            case = self._loadCase(id)
            case.restore()
            c.success_for   =   u'Fallakte wiederhergestellt!' 
            c.success_text  =   u'Bitte klicken Sie "OK", um fortzufahren.' 
            c.url_ok        =   h.url_for(controller="/case_overview")
            return render('/casemanagement/dialogs/success_restore.mako')
            #redirect_to(controller="/case_overview")
        else:
            c.context = "../main.mako" 
            c.confirm_for = RESTORE_CONFIRM 
            c.question = RESTORE_CONFIRM_TEXT 
            c.url_yes = "/case/restore/%s/1" % id
            c.url_no  = "/case_overview/"
            return render('/casemanagement/dialogs/confirm_restore.mako')

    @checkRole('cm_ka')
    def open(self, id, confirmed):
        id        = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        c.ds_id   = id
        c.form_navigation = self._getFormNavigation()
        if confirmed == 1:
            case = self._setState(id, 1)
            c.url_ok    = h.url_for(controller="/case", action="organisation", id=id)
            c.success_for   =   u'Fallakte geöffnet!'
            c.success_text  =   u'Die Fallakte wurde erfolgreich geöffnet.'
            return render('/casemanagement/dialogs/success_case_close.mako')
        else:
            c.context = "main.mako" 
            c.confirm_for = OPEN_CONFIRM 
            c.question = OPEN_CONFIRM_TEXT 
            c.url_yes = "/case/open/%s/1" % id
            c.url_no    = h.url_for(controller="/case", action="organisation", id=id)
            return render('/casemanagement/dialogs/confirm_open.mako')

    @checkRole('cm_ka')
    def close(self, id, confirmed):
        id        = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        c.ds_id   = id
        c.form_navigation = self._getFormNavigation()
        if confirmed == 1:
            case = self._setState(id, 2)
            c.url_ok    = h.url_for(controller="/case", action="organisation", id=id)
            c.success_for   =   u'Fallakte geschlossen!'
            c.success_text  =   u'Geschlossene Fallakten können Sie in der Fallaktenübersicht durch anklicken des Suchfilters "Geschlossen" einsehen.'
            return render('/casemanagement/dialogs/success_case_close.mako')
        else:
            c.context = "main.mako" 
            c.confirm_for = CLOSE_CONFIRM
            c.question = CLOSE_CONFIRM_TEXT 
            c.url_yes = "/case/close/%s/1" % id
            c.url_no    = h.url_for(controller="/case", action="organisation", id=id)
            return render('/casemanagement/dialogs/confirm_close.mako')

    @checkRole('cm_ka')
    def markForAnonymizeFromOverview(self, id, confirmed):
        try:
            id        = self._checkInt(id)
            confirmed = self._checkBool(confirmed)
            c.ds_id   = id
            case = load_case(id)
            # Check if the case is in a consistent state (all required fields
            # are filled)
            if not case.getState().phasesAreConsistent() and case.isYoungerThan(INCONSISTENCY_CHECK_AFTER):
                c.notification_for  = MARKANONYMIZE_INCOMPLETE_PHASE_FAILED 
                c.notification_text = MARKANONYMIZE_INCOMPLETE_PHASE_FAILED_TEXT
                c.url_ok            = h.url_for(controller="/case_overview")
                return render('/casemanagement/dialogs/failed_markanonymize_phase_incomplete_from_overview.mako')
            if confirmed == 1:
                state = case.getState().setState(4)
                if state:
                    c.success_for   =  MARKANONYMIZE_SUCCESS
                    c.success_text  =  MARKANONYMIZE_SUCCESS_TEXT
                    c.url_ok        =  h.url_for(controller="/case_overview")
                    return render('/casemanagement/dialogs/success_markanonymize.mako')
            else:
                c.confirm_for = ANONYMIZE_CONFIRM
                c.question = MARKANONYMIZE_CONFIRM_TEXT
                c.url_yes = "/case/markForAnonymize/%s/1" % id
                c.url_no  =  h.url_for(controller="/case_overview")
                return render('/casemanagement/dialogs/confirm_markanonymizeFromOverview.mako')
        except:
            return render('/tests/trace.mako')

    @checkRole('cm_ka')
    def markForAnonymize(self, id, confirmed):
        try:
            id        = self._checkInt(id)
            confirmed = self._checkBool(confirmed)
            c.ds_id   = id
            case = load_case(id)
            c.form_navigation = self._getFormNavigation()
            # Check if the case is in a consistent state (all required fields
            # are filled)
            if not case.getState().phasesAreConsistent() and case.isYoungerThan(INCONSISTENCY_CHECK_AFTER):
                c.notification_for  = MARKANONYMIZE_INCOMPLETE_PHASE_FAILED 
                c.notification_text = MARKANONYMIZE_INCOMPLETE_PHASE_FAILED_TEXT
                c.url_ok            = "/case/digest/%s" % id
                return render('/casemanagement/dialogs/failed_markanonymize_phase_incomplete.mako')
            if confirmed == 1:
                state = case.getState().setState(4)
                if state:
                    c.success_for   =  MARKANONYMIZE_SUCCESS
                    c.success_text  =  MARKANONYMIZE_SUCCESS_TEXT
                    c.url_ok        =  h.url_for(controller="/case_overview")
                    return render('/casemanagement/dialogs/success_markanonymize.mako')
            else:
                c.confirm_for = ANONYMIZE_CONFIRM
                c.question = MARKANONYMIZE_CONFIRM_TEXT
                c.url_yes = "/case/markForAnonymize/%s/1" % id
                c.url_no  = "/case/digest/%s" % id
                return render('/casemanagement/dialogs/confirm_markanonymize.mako')
        except:
            return render('/tests/trace.mako')

    @checkRole('admin_ka')
    def anonymize(self, id, confirmed):
        id        = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        c.ds_id   = id
        #c.form_navigation = self._getFormNavigation()
        if confirmed == 1:
            case = self._loadCase(id)
            if case.anonymize():
                c.success_for   =  ANONYMIZE_SUCCESS
                c.success_text  =  ANONYMIZE_SUCCESS_TEXT
                c.url_ok        =  h.url_for(controller="/case_overview")
                return render('/casemanagement/dialogs/success_anonymize.mako')
            # TODO: Create Errorpage
            return "Error! Could not anonymize ds."
        else:
            c.context = "../main.mako"
            c.confirm_for = ANONYMIZE_CONFIRM
            c.question = ANONYMIZE_CONFIRM_TEXT
            c.url_yes = "/case/anonymize/%s/1" % id
            c.url_no  = "/case/digest/%s" % id
            return render('/casemanagement/dialogs/confirm_anonymize.mako')

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
