# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
# Sascha L. Teichmann <teichmann@intevation.de>
#

import logging

from waskaweb.lib.base     import *
from waskaweb.lib.search   import INCONSISTENCY_CHECK_AFTER 
from waskaweb.lib.security import checkLogin, userIdentity, generateID, checkRole
from waskaweb.model.user   import UserObject
from waskaweb.model.case   import CaseOverview
from waskaweb.model.news   import NewsList
from waskaweb.model.appointment import MaxSaveTimeReminderOverview, \
                                       CaseAppointmentOverview, \
                                       GlobalAppointmentOverview, \
                                       CaseReminderOverview

import md5
from datetime import datetime
from sets import Set

log = logging.getLogger(__name__)

class WaskaController(BaseController):

    def index(self):
        redirect_to(action='start')

    @checkRole(('admin_ka', 'cm_ka', 'pb_ka', 'pt_dlr'))
    def start(self):
        user = session['USER_AUTHORIZED']
        if config.get('evaluation_server', '0') == '1': 
        # On evaluation server do not try to find out which cases are saved
        # longer than allowed. This would be a real performance killer and
        # cases are anonymized anyway
            pass
        else:
            # Get list of cases which were not edited longer than the allowed max
            # savetime
            maxsavetimelist   = MaxSaveTimeReminderOverview()
            caseremindlist        = CaseReminderOverview()
            #if h.hasRole(['cm_ka']):
            #    remindlist.join(maxsavetimelist)
            #    remindlist.sort()
            #    c.remindlist  = remindlist
            #else:
            #    c.remindlist  = maxsavetimelist 
            c.remindlist  = maxsavetimelist 
            # Load case and global appointments
            caseapp           = CaseAppointmentOverview()
            globalapp         = GlobalAppointmentOverview()
            # As we want to display both kind of app. we need to join the lists and
            # sort them
            if h.hasRole(['cm_ka']):
                caseapp.join(caseremindlist)
                caseapp.sort()
                c.appointmentlist = caseapp
            c.globalappointmentlist = globalapp

            # LOAD NEWS
            c.news = NewsList(user)

        #find out if there are cases with errors

        #Search inconsistencys on dates
        search_str1 = "state:1;state:2;state:4;state:5;bad:1;bad:2;bad:3;bad:4;bad:5;bad:6;bad:7;own:%s" % user.id

        #Search inconsistencys on phases but ignore older cases. 
        search_str2 = "state:1;state:2;state:4;bad:clearing_start;bad:clearing_ende;bad:beratung_start;bad:beratung_ende;bad:cm_start;bad:cm_ende;bad:nachbetreuung_start;bad:nachbetreuung_ende;only_cases_after:'%s';own:%s" % (INCONSISTENCY_CHECK_AFTER, user.id)

        error_cases1 = Set(c.id for c in CaseOverview().search(search_str1))
        error_cases2 = Set(c.id for c in CaseOverview().search(search_str2))
        c.num_error_cases = len(error_cases1.union(error_cases2))

        if c.num_error_cases > 0:
            session['hascaseerrors'] = True
        else:
            session['hascaseerrors'] = False
        session.save()
        return render('/start/start.mako')

    def help(self):
        return render('/start/help.mako') 

    def settings(self, id):
        id = self._checkInt(id)
        redirect_to(controller="usersettings", action="showSettings", id=id)

    def info(self):
        return render('/start/responsible.mako') 

    def logout(self):
        # XXX: invalidate() does not delete backend data :-/
        session.delete()
        session.save()
        session.invalidate()
        response.delete_cookie('waska_auth')
        return render('/logout.mako')

    def login(self):
        return render('/login.mako')

    def loginAction(self):
        try:
            login  = request.params.getone('username')
            passwd = request.params.getone('password')
            user   = checkLogin(login, passwd)
            if user:
                c.login_failed = False
                shared = generateID()
                session['USER_AUTHORIZED'] = user
                session['AUTH'] = md5.new(shared + userIdentity()).digest()
                session.save()
                response.set_cookie('waska_auth', shared, secure=True)
                redirect_to(action="start")
            else:
                c.login_failed = True
                return self.login()
        except KeyError:
            pass
        return self.login()

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
