# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
"""Helper functions

Consists of functions to typically be used within templates, but also
available to Controllers. This module is available to both as 'h'.
"""
#from waskaweb.lib.base import *
import locale
import sys
import re
from pylons import c, cache, config, g, request, response, session
from webhelpers import *

import datetime

VALID_DATE = re.compile(r'^([0-9]{1,2})\.([0-9]{1,2})\.([0-9]{4})$')
VALID_TIME = re.compile(r'^([0-9]{1,2}):([0-9]{2})')

def hasRole(rolelist):
    try:
        user = session['USER_AUTHORIZED']
        return user.getPermissionRole() in rolelist
    except KeyError:
        return False

def getRole():
    try:
        user = session['USER_AUTHORIZED']
        return user.getRole()
    except KeyError:
        return ''

def getUserLastname():
    try:
        return session['USER_AUTHORIZED'].last_name
    except KeyError:
        return ''

def getUserFullname():
    try:
        user = session['USER_AUTHORIZED']
        return "%s, %s" % (user.last_name, user.first_name) 
    except KeyError:
        return ''

def getCaseShort(name, n):
    if len(name) < n:
         return name
    else:
         n = n-3
         return "%s..." % (name[:n])

def getLogin():
    try:
        user = session['USER_AUTHORIZED']
        return user.login 
    except KeyError:
        return ''

def getUserId():
    try:
        user = session['USER_AUTHORIZED']
        return user.id 
    except KeyError:
        return ''
    

def formatNumber(number):
    retval = locale.format("%.2f",(number),1)
    return retval

def dd_mm_YYYY(d, empty = ''):
    ''' datetime.strftime() dislikes years before 1900. '''
    if d is None: return empty
    return "%02d.%02d.%d" % (d.day, d.month, d.year)

def HH_MM(d):
    ''' datetime.strftime() dislikes years before 1900. '''
    if d is None: return ''
    if type(d) is datetime.date:
        # not so smart
        d = datetime.datetime(d.year, d.month, d.day)
    return "%02d:%02d" % (d.hour, d.minute)

def dd_mm_yyyy_HH_MM(d, empty = ''):
    ''' datetime.strftime() dislikes years before 1900. '''
    if d is None: return empty
    if type(d) is datetime.date:
        # not so smart
        d = datetime.datetime(d.year, d.month, d.day)
    return "%02d.%02d.%d %02d:%02d" % (d.day, d.month, d.year, d.hour, d.minute)

def status_message(s):
    return "%s: %s" % (dd_mm_yyyy_HH_MM(datetime.datetime.now()), s)

def timedelta_in_minutes(ti):
    if ti is None: return None
    days = float(ti.days * (24*60))
    secs = ti.seconds / 60.0
    return round(days + secs)

def timedelta_in_weeks(ti, empty = None):
    if ti is None: return empty
    days = float(ti.days) / 7.0
    secs = ti.seconds / (60.0 * 24.0 * 7.0)
    return formatNumber(days + secs)

def human_timedelta(td, empty=None):
    if td is None: return empty
    out = []
    if -td == abs(td):
        out.append("-")
        td = -td
    days = td.days
    if days == 1:
        out.append("1 Tag,")
    if days > 1:
        out.append("%d Tage," % days)

    secs  = td.seconds
    hours = secs / 3600
    secs -= hours * 3600
    mins  = secs / 60
    secs -= mins * 60

    out.append("%d:%02d:%02d" % (hours, mins, secs))

    return " ".join(out)

def checkDate(value):
    m = VALID_DATE.match(value)
    if not m: return None
    try:
        return datetime.datetime(int(m.group(3)), int(m.group(2)), int(m.group(1)))
    except ValueError:
        return None

def checkTime(value):
    m = VALID_TIME.match(value)
    if not m: return None
    return datetime.timedelta(hours = int(m.group(1)), minutes = int(m.group(2)))

def getFormularHeaders(page_id):
    navigation = session.get('navigation.tree')
    try:
        headers = navigation.getHeaders(page_id)
    except StandardError, err:
        print err
        return "" 
    return headers

def getHelp(help_id):
    from waskaweb.lib.renderer import FormData
    formdata = FormData(g.helpData)
    return formdata.getHelp(help_id)

def getFormularName(page_id):
    navigation = session.get('navigation.tree')
    try:
        ti = navigation.getTreeItem(page_id)
        return "Akte: %s" % ti.description
    except:
        pass

def getFormularErrors(page_id):
    errors_on_page = {}
    # Get treeitem
    navigation = session.get('navigation.tree')
    try:
        ti = navigation.getTreeItem(page_id)
    except:
        return {} 
    errors = session.get('case').getFormErrors()
    try:
        for key, err in errors.iteritems(): 
            if err.page == ti.page:
                errors_on_page[key] = err
    except:
        pass
    return errors_on_page

def getPrintUrl(print_form, ds_id, page_id):
    if print_form == "digest":
        return "/case/printout/%s/0/digest" % (ds_id)
    elif print_form == "show":
        return "/case/printout/%s/%s/show" % (ds_id, page_id)
    else: 
        return "#"

def slashSplit(s):
    out = []
    for p in s.split("/"):
        if p.find("=") == -1 and out:
            out[-1] += '/'+ p
        else:
            out.append(p)
    return out

def getKAName():
    kaname = ""
    try:
        for f in slashSplit(request.environ['SSL_CLIENT_S_DN'])[::-1]:
            field = f.split('=')
            # There are two OU. We want the one != WASKA
            if field[0] == "OU" and field[1] != "WASKA":
                kaname = field[1]
        return kaname
    except:
        print >> sys.stderr, "Could not fetch KA-name from client certificate"
    return ''
